# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'stereoprojUI.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets
import os
fn = os.path.join(os.path.dirname(__file__), "stereo-icon.png")

class Ui_StereoProj(object):
    def setupUi(self, StereoProj):
        StereoProj.setObjectName("StereoProj")
        StereoProj.resize(1403, 884)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(fn), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        StereoProj.setWindowIcon(icon)
        StereoProj.setIconSize(QtCore.QSize(24, 24))
        self.centralwidget = QtWidgets.QWidget(StereoProj)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.mplvl = QtWidgets.QGridLayout()
        self.mplvl.setObjectName("mplvl")
        self.gridLayout_7.addLayout(self.mplvl, 0, 0, 1, 1)
        self.groupBox_6 = QtWidgets.QGroupBox(self.centralwidget)
        self.groupBox_6.setMaximumSize(QtCore.QSize(460, 16777215))
        self.groupBox_6.setTitle("")
        self.groupBox_6.setObjectName("groupBox_6")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_6)
        self.gridLayout.setContentsMargins(-1, 0, -1, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox_2 = QtWidgets.QGroupBox(self.groupBox_6)
        self.groupBox_2.setEnabled(True)
        self.groupBox_2.setMaximumSize(QtCore.QSize(230, 16777215))
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_3.setContentsMargins(5, 5, 5, 5)
        self.gridLayout_3.setSpacing(5)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.rot_gm_button = QtWidgets.QPushButton(self.groupBox_2)
        self.rot_gm_button.setObjectName("rot_gm_button")
        self.gridLayout_3.addWidget(self.rot_gm_button, 9, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_3.addItem(spacerItem, 11, 1, 1, 1)
        self.diff_entry = QtWidgets.QLineEdit(self.groupBox_2)
        self.diff_entry.setObjectName("diff_entry")
        self.gridLayout_3.addWidget(self.diff_entry, 0, 1, 1, 3)
        self.diff_label = QtWidgets.QLabel(self.groupBox_2)
        self.diff_label.setObjectName("diff_label")
        self.gridLayout_3.addWidget(self.diff_label, 0, 0, 1, 1)
        self.rot_g_entry = QtWidgets.QLineEdit(self.groupBox_2)
        self.rot_g_entry.setObjectName("rot_g_entry")
        self.gridLayout_3.addWidget(self.rot_g_entry, 9, 1, 1, 1)
        self.rot_diff_label = QtWidgets.QLabel(self.groupBox_2)
        self.rot_diff_label.setObjectName("rot_diff_label")
        self.gridLayout_3.addWidget(self.rot_diff_label, 8, 0, 1, 3)
        self.button_trace = QtWidgets.QPushButton(self.groupBox_2)
        self.button_trace.setObjectName("button_trace")
        self.gridLayout_3.addWidget(self.button_trace, 5, 0, 1, 2)
        self.rot_gp_button = QtWidgets.QPushButton(self.groupBox_2)
        self.rot_gp_button.setObjectName("rot_gp_button")
        self.gridLayout_3.addWidget(self.rot_gp_button, 9, 2, 1, 1)
        self.tilt_entry = QtWidgets.QLineEdit(self.groupBox_2)
        self.tilt_entry.setObjectName("tilt_entry")
        self.gridLayout_3.addWidget(self.tilt_entry, 1, 1, 1, 3)
        self.rg_label = QtWidgets.QLabel(self.groupBox_2)
        self.rg_label.setText("")
        self.rg_label.setObjectName("rg_label")
        self.gridLayout_3.addWidget(self.rg_label, 10, 1, 1, 1)
        self.tilt_label = QtWidgets.QLabel(self.groupBox_2)
        self.tilt_label.setObjectName("tilt_label")
        self.gridLayout_3.addWidget(self.tilt_label, 1, 0, 1, 1)
        self.inclination_label = QtWidgets.QLabel(self.groupBox_2)
        self.inclination_label.setObjectName("inclination_label")
        self.gridLayout_3.addWidget(self.inclination_label, 4, 0, 1, 2)
        self.inclinaison_entry = QtWidgets.QLineEdit(self.groupBox_2)
        self.inclinaison_entry.setObjectName("inclinaison_entry")
        self.gridLayout_3.addWidget(self.inclinaison_entry, 4, 2, 1, 2)
        self.gridLayout.addWidget(self.groupBox_2, 2, 0, 1, 1)
        self.groupBox_5 = QtWidgets.QGroupBox(self.groupBox_6)
        self.groupBox_5.setMaximumSize(QtCore.QSize(200, 16777215))
        self.groupBox_5.setObjectName("groupBox_5")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBox_5)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.coord_label = QtWidgets.QLabel(self.groupBox_5)
        self.coord_label.setText("")
        self.coord_label.setObjectName("coord_label")
        self.gridLayout_6.addWidget(self.coord_label, 5, 0, 1, 3)
        self.lab_euler2 = QtWidgets.QLabel(self.groupBox_5)
        self.lab_euler2.setObjectName("lab_euler2")
        self.gridLayout_6.addWidget(self.lab_euler2, 2, 0, 1, 3)
        self.button_trace2 = QtWidgets.QPushButton(self.groupBox_5)
        self.button_trace2.setObjectName("button_trace2")
        self.gridLayout_6.addWidget(self.button_trace2, 1, 0, 1, 3)
        self.lab_coord = QtWidgets.QLabel(self.groupBox_5)
        self.lab_coord.setObjectName("lab_coord")
        self.gridLayout_6.addWidget(self.lab_coord, 4, 0, 1, 3)
        self.phi1phiphi2_entry = QtWidgets.QLineEdit(self.groupBox_5)
        self.phi1phiphi2_entry.setObjectName("phi1phiphi2_entry")
        self.gridLayout_6.addWidget(self.phi1phiphi2_entry, 0, 0, 1, 3)
        self.angle_euler_label = QtWidgets.QLabel(self.groupBox_5)
        self.angle_euler_label.setText("")
        self.angle_euler_label.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByMouse)
        self.angle_euler_label.setObjectName("angle_euler_label")
        self.gridLayout_6.addWidget(self.angle_euler_label, 3, 0, 1, 3)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_6.addItem(spacerItem1, 6, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_5, 2, 1, 1, 1)
        self.groupBox_9 = QtWidgets.QGroupBox(self.groupBox_6)
        self.groupBox_9.setMaximumSize(QtCore.QSize(230, 16777215))
        self.groupBox_9.setObjectName("groupBox_9")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.groupBox_9)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.uvw_button = QtWidgets.QCheckBox(self.groupBox_9)
        self.uvw_button.setObjectName("uvw_button")
        self.gridLayout_10.addWidget(self.uvw_button, 14, 3, 1, 1)
        self.hexa_button = QtWidgets.QCheckBox(self.groupBox_9)
        self.hexa_button.setObjectName("hexa_button")
        self.gridLayout_10.addWidget(self.hexa_button, 14, 0, 1, 1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_10.addItem(spacerItem2, 17, 0, 1, 1)
        self.color_trace_bleu = QtWidgets.QRadioButton(self.groupBox_9)
        self.color_trace_bleu.setObjectName("color_trace_bleu")
        self.gridLayout_10.addWidget(self.color_trace_bleu, 0, 0, 1, 1)
        self.color_trace_vert = QtWidgets.QRadioButton(self.groupBox_9)
        self.color_trace_vert.setObjectName("color_trace_vert")
        self.gridLayout_10.addWidget(self.color_trace_vert, 0, 1, 1, 2)
        self.size_var = QtWidgets.QLineEdit(self.groupBox_9)
        self.size_var.setObjectName("size_var")
        self.gridLayout_10.addWidget(self.size_var, 6, 3, 1, 1)
        self.text_size_label = QtWidgets.QLabel(self.groupBox_9)
        self.text_size_label.setObjectName("text_size_label")
        self.gridLayout_10.addWidget(self.text_size_label, 7, 0, 1, 2)
        self.color_trace_rouge = QtWidgets.QRadioButton(self.groupBox_9)
        self.color_trace_rouge.setObjectName("color_trace_rouge")
        self.gridLayout_10.addWidget(self.color_trace_rouge, 0, 3, 1, 1)
        self.text_size_entry = QtWidgets.QLineEdit(self.groupBox_9)
        self.text_size_entry.setObjectName("text_size_entry")
        self.gridLayout_10.addWidget(self.text_size_entry, 7, 3, 1, 1)
        self.size_var_label = QtWidgets.QLabel(self.groupBox_9)
        self.size_var_label.setObjectName("size_var_label")
        self.gridLayout_10.addWidget(self.size_var_label, 6, 0, 1, 3)
        self.tilt_angle_label = QtWidgets.QLabel(self.groupBox_9)
        self.tilt_angle_label.setObjectName("tilt_angle_label")
        self.gridLayout_10.addWidget(self.tilt_angle_label, 12, 0, 1, 3)
        self.label_2 = QtWidgets.QLabel(self.groupBox_9)
        self.label_2.setObjectName("label_2")
        self.gridLayout_10.addWidget(self.label_2, 13, 0, 1, 3)
        self.tilt_angle_entry = QtWidgets.QLineEdit(self.groupBox_9)
        self.tilt_angle_entry.setObjectName("tilt_angle_entry")
        self.gridLayout_10.addWidget(self.tilt_angle_entry, 12, 3, 1, 1)
        self.image_angle_entry = QtWidgets.QLineEdit(self.groupBox_9)
        self.image_angle_entry.setObjectName("image_angle_entry")
        self.gridLayout_10.addWidget(self.image_angle_entry, 13, 3, 1, 1)
        self.reset_view_button = QtWidgets.QPushButton(self.groupBox_9)
        self.reset_view_button.setObjectName("reset_view_button")
        self.gridLayout_10.addWidget(self.reset_view_button, 15, 0, 1, 4)
        self.real_space_checkBox = QtWidgets.QCheckBox(self.groupBox_9)
        self.real_space_checkBox.setObjectName("real_space_checkBox")
        self.gridLayout_10.addWidget(self.real_space_checkBox, 8, 0, 1, 2)
        self.style_box = QtWidgets.QCheckBox(self.groupBox_9)
        self.style_box.setObjectName("style_box")
        self.gridLayout_10.addWidget(self.style_box, 2, 0, 1, 2)
        self.wulff_button = QtWidgets.QCheckBox(self.groupBox_9)
        self.wulff_button.setObjectName("wulff_button")
        self.gridLayout_10.addWidget(self.wulff_button, 2, 2, 1, 2)
        self.gridLayout.addWidget(self.groupBox_9, 1, 0, 1, 1)
        self.groupBox_4 = QtWidgets.QGroupBox(self.groupBox_6)
        self.groupBox_4.setMaximumSize(QtCore.QSize(200, 16777215))
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.trace_plan_sym_button = QtWidgets.QPushButton(self.groupBox_4)
        self.trace_plan_sym_button.setObjectName("trace_plan_sym_button")
        self.gridLayout_5.addWidget(self.trace_plan_sym_button, 4, 0, 1, 3)
        self.sym_button = QtWidgets.QPushButton(self.groupBox_4)
        self.sym_button.setObjectName("sym_button")
        self.gridLayout_5.addWidget(self.sym_button, 2, 0, 1, 3)
        self.undo_trace_cone_button = QtWidgets.QPushButton(self.groupBox_4)
        self.undo_trace_cone_button.setObjectName("undo_trace_cone_button")
        self.gridLayout_5.addWidget(self.undo_trace_cone_button, 6, 3, 1, 1)
        self.trace_plan_button = QtWidgets.QPushButton(self.groupBox_4)
        self.trace_plan_button.setObjectName("trace_plan_button")
        self.gridLayout_5.addWidget(self.trace_plan_button, 3, 0, 1, 3)
        self.undo_trace_plan_sym_button = QtWidgets.QPushButton(self.groupBox_4)
        self.undo_trace_plan_sym_button.setObjectName("undo_trace_plan_sym_button")
        self.gridLayout_5.addWidget(self.undo_trace_plan_sym_button, 4, 3, 1, 1)
        self.inclination_entry = QtWidgets.QLineEdit(self.groupBox_4)
        self.inclination_entry.setObjectName("inclination_entry")
        self.gridLayout_5.addWidget(self.inclination_entry, 6, 2, 1, 1)
        self.trace_cone_button = QtWidgets.QPushButton(self.groupBox_4)
        self.trace_cone_button.setObjectName("trace_cone_button")
        self.gridLayout_5.addWidget(self.trace_cone_button, 6, 0, 1, 2)
        self.undo_trace_schmid = QtWidgets.QPushButton(self.groupBox_4)
        self.undo_trace_schmid.setObjectName("undo_trace_schmid")
        self.gridLayout_5.addWidget(self.undo_trace_schmid, 5, 3, 1, 1)
        self.norm_button = QtWidgets.QPushButton(self.groupBox_4)
        self.norm_button.setObjectName("norm_button")
        self.gridLayout_5.addWidget(self.norm_button, 7, 0, 1, 1)
        self.pole_entry = QtWidgets.QLineEdit(self.groupBox_4)
        self.pole_entry.setObjectName("pole_entry")
        self.gridLayout_5.addWidget(self.pole_entry, 0, 0, 1, 4)
        self.undo_trace_plan_button = QtWidgets.QPushButton(self.groupBox_4)
        self.undo_trace_plan_button.setObjectName("undo_trace_plan_button")
        self.gridLayout_5.addWidget(self.undo_trace_plan_button, 3, 3, 1, 1)
        self.undo_addpole_button = QtWidgets.QPushButton(self.groupBox_4)
        self.undo_addpole_button.setObjectName("undo_addpole_button")
        self.gridLayout_5.addWidget(self.undo_addpole_button, 1, 3, 1, 1)
        self.addpole_button = QtWidgets.QPushButton(self.groupBox_4)
        self.addpole_button.setObjectName("addpole_button")
        self.gridLayout_5.addWidget(self.addpole_button, 1, 0, 1, 3)
        self.dhkl_label = QtWidgets.QLabel(self.groupBox_4)
        self.dhkl_label.setText("")
        self.dhkl_label.setObjectName("dhkl_label")
        self.gridLayout_5.addWidget(self.dhkl_label, 7, 2, 1, 2)
        self.undo_sym_button = QtWidgets.QPushButton(self.groupBox_4)
        self.undo_sym_button.setObjectName("undo_sym_button")
        self.gridLayout_5.addWidget(self.undo_sym_button, 2, 3, 1, 1)
        self.trace_schmid_button = QtWidgets.QPushButton(self.groupBox_4)
        self.trace_schmid_button.setObjectName("trace_schmid_button")
        self.gridLayout_5.addWidget(self.trace_schmid_button, 5, 0, 1, 3)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_5.addItem(spacerItem3, 8, 2, 1, 1)
        self.gridLayout.addWidget(self.groupBox_4, 1, 1, 1, 1)
        self.groupBox_3 = QtWidgets.QGroupBox(self.groupBox_6)
        self.groupBox_3.setMaximumSize(QtCore.QSize(200, 16777215))
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_4.setContentsMargins(5, 5, 5, 5)
        self.gridLayout_4.setSpacing(5)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.angle_beta_entry = QtWidgets.QLineEdit(self.groupBox_3)
        self.angle_beta_entry.setObjectName("angle_beta_entry")
        self.gridLayout_4.addWidget(self.angle_beta_entry, 8, 2, 1, 1)
        self.angle_beta_label = QtWidgets.QLabel(self.groupBox_3)
        self.angle_beta_label.setObjectName("angle_beta_label")
        self.gridLayout_4.addWidget(self.angle_beta_label, 8, 0, 1, 1)
        self.angle_z_buttonm = QtWidgets.QPushButton(self.groupBox_3)
        self.angle_z_buttonm.setObjectName("angle_z_buttonm")
        self.gridLayout_4.addWidget(self.angle_z_buttonm, 11, 1, 1, 1)
        self.angle_alpha_label_2 = QtWidgets.QLabel(self.groupBox_3)
        self.angle_alpha_label_2.setText("")
        self.angle_alpha_label_2.setObjectName("angle_alpha_label_2")
        self.gridLayout_4.addWidget(self.angle_alpha_label_2, 2, 5, 1, 1)
        self.angle_alpha_entry = QtWidgets.QLineEdit(self.groupBox_3)
        self.angle_alpha_entry.setObjectName("angle_alpha_entry")
        self.gridLayout_4.addWidget(self.angle_alpha_entry, 2, 2, 1, 1)
        self.angle_beta_buttonp = QtWidgets.QPushButton(self.groupBox_3)
        self.angle_beta_buttonp.setObjectName("angle_beta_buttonp")
        self.gridLayout_4.addWidget(self.angle_beta_buttonp, 8, 3, 1, 1)
        self.label = QtWidgets.QLabel(self.groupBox_3)
        self.label.setObjectName("label")
        self.gridLayout_4.addWidget(self.label, 0, 4, 1, 2)
        self.angle_beta_buttonm = QtWidgets.QPushButton(self.groupBox_3)
        self.angle_beta_buttonm.setObjectName("angle_beta_buttonm")
        self.gridLayout_4.addWidget(self.angle_beta_buttonm, 8, 1, 1, 1)
        self.angle_z_label = QtWidgets.QLabel(self.groupBox_3)
        self.angle_z_label.setObjectName("angle_z_label")
        self.gridLayout_4.addWidget(self.angle_z_label, 11, 0, 1, 1)
        self.angle_z_entry = QtWidgets.QLineEdit(self.groupBox_3)
        self.angle_z_entry.setObjectName("angle_z_entry")
        self.gridLayout_4.addWidget(self.angle_z_entry, 11, 2, 1, 1)
        self.lock_checkButton = QtWidgets.QCheckBox(self.groupBox_3)
        self.lock_checkButton.setObjectName("lock_checkButton")
        self.gridLayout_4.addWidget(self.lock_checkButton, 0, 0, 1, 4)
        self.angle_alpha_buttonm = QtWidgets.QPushButton(self.groupBox_3)
        self.angle_alpha_buttonm.setObjectName("angle_alpha_buttonm")
        self.gridLayout_4.addWidget(self.angle_alpha_buttonm, 2, 1, 1, 1)
        self.beta_signBox = QtWidgets.QCheckBox(self.groupBox_3)
        self.beta_signBox.setText("")
        self.beta_signBox.setObjectName("beta_signBox")
        self.gridLayout_4.addWidget(self.beta_signBox, 8, 4, 1, 1)
        self.angle_z_buttonp = QtWidgets.QPushButton(self.groupBox_3)
        self.angle_z_buttonp.setObjectName("angle_z_buttonp")
        self.gridLayout_4.addWidget(self.angle_z_buttonp, 11, 3, 1, 1)
        self.alpha_signBox = QtWidgets.QCheckBox(self.groupBox_3)
        self.alpha_signBox.setText("")
        self.alpha_signBox.setObjectName("alpha_signBox")
        self.gridLayout_4.addWidget(self.alpha_signBox, 2, 4, 1, 1)
        self.theta_signBox = QtWidgets.QCheckBox(self.groupBox_3)
        self.theta_signBox.setText("")
        self.theta_signBox.setObjectName("theta_signBox")
        self.gridLayout_4.addWidget(self.theta_signBox, 11, 4, 1, 1)
        self.angle_alpha_buttonp = QtWidgets.QPushButton(self.groupBox_3)
        self.angle_alpha_buttonp.setObjectName("angle_alpha_buttonp")
        self.gridLayout_4.addWidget(self.angle_alpha_buttonp, 2, 3, 1, 1)
        self.angle_beta_label_2 = QtWidgets.QLabel(self.groupBox_3)
        self.angle_beta_label_2.setText("")
        self.angle_beta_label_2.setObjectName("angle_beta_label_2")
        self.gridLayout_4.addWidget(self.angle_beta_label_2, 8, 5, 1, 1)
        self.angle_alpha_label = QtWidgets.QLabel(self.groupBox_3)
        self.angle_alpha_label.setObjectName("angle_alpha_label")
        self.gridLayout_4.addWidget(self.angle_alpha_label, 2, 0, 1, 1)
        self.angle_z_label_2 = QtWidgets.QLabel(self.groupBox_3)
        self.angle_z_label_2.setText("")
        self.angle_z_label_2.setObjectName("angle_z_label_2")
        self.gridLayout_4.addWidget(self.angle_z_label_2, 11, 5, 1, 1)
        self.reset_angle_button = QtWidgets.QPushButton(self.groupBox_3)
        self.reset_angle_button.setObjectName("reset_angle_button")
        self.gridLayout_4.addWidget(self.reset_angle_button, 12, 0, 1, 2)
        self.gridLayout.addWidget(self.groupBox_3, 0, 1, 1, 1)
        self.groupBox = QtWidgets.QGroupBox(self.groupBox_6)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        self.groupBox.setMaximumSize(QtCore.QSize(230, 16777215))
        self.groupBox.setObjectName("groupBox")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox)
        self.gridLayout_2.setContentsMargins(5, 5, 5, 5)
        self.gridLayout_2.setSpacing(5)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.abc_entry = QtWidgets.QLineEdit(self.groupBox)
        self.abc_entry.setObjectName("abc_entry")
        self.gridLayout_2.addWidget(self.abc_entry, 0, 3, 1, 3)
        self.alphabetagamma_entry = QtWidgets.QLineEdit(self.groupBox)
        self.alphabetagamma_entry.setObjectName("alphabetagamma_entry")
        self.gridLayout_2.addWidget(self.alphabetagamma_entry, 2, 3, 1, 3)
        self.alphabetagamma_label = QtWidgets.QLabel(self.groupBox)
        self.alphabetagamma_label.setObjectName("alphabetagamma_label")
        self.gridLayout_2.addWidget(self.alphabetagamma_label, 2, 0, 1, 1)
        self.e_entry = QtWidgets.QLineEdit(self.groupBox)
        self.e_entry.setObjectName("e_entry")
        self.gridLayout_2.addWidget(self.e_entry, 4, 4, 1, 1)
        self.abc_label = QtWidgets.QLabel(self.groupBox)
        self.abc_label.setObjectName("abc_label")
        self.gridLayout_2.addWidget(self.abc_label, 0, 0, 1, 1)
        self.e_label = QtWidgets.QLabel(self.groupBox)
        self.e_label.setObjectName("e_label")
        self.gridLayout_2.addWidget(self.e_label, 4, 0, 1, 4)
        self.d_label = QtWidgets.QLabel(self.groupBox)
        self.d_label.setObjectName("d_label")
        self.gridLayout_2.addWidget(self.d_label, 9, 0, 1, 1)
        self.d_label_var = QtWidgets.QLabel(self.groupBox)
        self.d_label_var.setText("")
        self.d_label_var.setObjectName("d_label_var")
        self.gridLayout_2.addWidget(self.d_label_var, 10, 2, 1, 3)
        self.reciprocal_checkBox = QtWidgets.QCheckBox(self.groupBox)
        self.reciprocal_checkBox.setObjectName("reciprocal_checkBox")
        self.gridLayout_2.addWidget(self.reciprocal_checkBox, 5, 0, 1, 5)
        self.space_group_Box = QtWidgets.QComboBox(self.groupBox)
        self.space_group_Box.setObjectName("space_group_Box")
        self.gridLayout_2.addWidget(self.space_group_Box, 7, 0, 1, 6)
        self.d_Slider = QtWidgets.QSlider(self.groupBox)
        self.d_Slider.setOrientation(QtCore.Qt.Horizontal)
        self.d_Slider.setObjectName("d_Slider")
        self.gridLayout_2.addWidget(self.d_Slider, 9, 2, 1, 4)
        self.gridLayout.addWidget(self.groupBox, 0, 0, 1, 1)
        self.gridLayout_7.addWidget(self.groupBox_6, 0, 1, 1, 1)
        StereoProj.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(StereoProj)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1403, 23))
        self.menubar.setObjectName("menubar")
        self.menuSave = QtWidgets.QMenu(self.menubar)
        self.menuSave.setObjectName("menuSave")
        self.menuStructure = QtWidgets.QMenu(self.menubar)
        self.menuStructure.setObjectName("menuStructure")
        self.menuAngle = QtWidgets.QMenu(self.menubar)
        self.menuAngle.setObjectName("menuAngle")
        self.menuSchmid_factor = QtWidgets.QMenu(self.menubar)
        self.menuSchmid_factor.setObjectName("menuSchmid_factor")
        self.menuXyz_directions = QtWidgets.QMenu(self.menubar)
        self.menuXyz_directions.setObjectName("menuXyz_directions")
        self.menuWidth = QtWidgets.QMenu(self.menubar)
        self.menuWidth.setObjectName("menuWidth")
        self.menuIntersections = QtWidgets.QMenu(self.menubar)
        self.menuIntersections.setObjectName("menuIntersections")
        self.menuDiffraction = QtWidgets.QMenu(self.menubar)
        self.menuDiffraction.setObjectName("menuDiffraction")
        self.menuList = QtWidgets.QMenu(self.menubar)
        self.menuList.setObjectName("menuList")
        self.menuIPF = QtWidgets.QMenu(self.menubar)
        self.menuIPF.setObjectName("menuIPF")
        StereoProj.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(StereoProj)
        self.statusbar.setObjectName("statusbar")
        StereoProj.setStatusBar(self.statusbar)
        self.actionSave_figure = QtWidgets.QAction(StereoProj)
        self.actionSave_figure.setObjectName("actionSave_figure")
        self.actionCalculate_Schmid_factor = QtWidgets.QAction(StereoProj)
        self.actionCalculate_Schmid_factor.setObjectName("actionCalculate_Schmid_factor")
        self.actionCalculate_angle = QtWidgets.QAction(StereoProj)
        self.actionCalculate_angle.setObjectName("actionCalculate_angle")
        self.actionCalculate_xyz = QtWidgets.QAction(StereoProj)
        self.actionCalculate_xyz.setObjectName("actionCalculate_xyz")
        self.actionCalculate_apparent_width = QtWidgets.QAction(StereoProj)
        self.actionCalculate_apparent_width.setObjectName("actionCalculate_apparent_width")
        self.actionPlanes = QtWidgets.QAction(StereoProj)
        self.actionPlanes.setObjectName("actionPlanes")
        self.actionProj_directions = QtWidgets.QAction(StereoProj)
        self.actionProj_directions.setObjectName("actionProj_directions")
        self.actionPlane_cone = QtWidgets.QAction(StereoProj)
        self.actionPlane_cone.setObjectName("actionPlane_cone")
        self.actionCalculate_intersections = QtWidgets.QAction(StereoProj)
        self.actionCalculate_intersections.setObjectName("actionCalculate_intersections")
        self.actionHkl_uvw = QtWidgets.QAction(StereoProj)
        self.actionHkl_uvw.setObjectName("actionHkl_uvw")
        self.actionPlot_Kikuchi_lines = QtWidgets.QAction(StereoProj)
        self.actionPlot_Kikuchi_lines.setObjectName("actionPlot_Kikuchi_lines")
        self.actionShow_list_of_poles_directions = QtWidgets.QAction(StereoProj)
        self.actionShow_list_of_poles_directions.setObjectName("actionShow_list_of_poles_directions")
        self.actiondraw_IPF = QtWidgets.QAction(StereoProj)
        self.actiondraw_IPF.setObjectName("actiondraw_IPF")
        self.menuSave.addAction(self.actionSave_figure)
        self.menuAngle.addAction(self.actionCalculate_angle)
        self.menuSchmid_factor.addAction(self.actionCalculate_Schmid_factor)
        self.menuXyz_directions.addAction(self.actionCalculate_xyz)
        self.menuXyz_directions.addAction(self.actionHkl_uvw)
        self.menuWidth.addAction(self.actionCalculate_apparent_width)
        self.menuIntersections.addAction(self.actionCalculate_intersections)
        self.menuDiffraction.addAction(self.actionPlot_Kikuchi_lines)
        self.menuList.addAction(self.actionShow_list_of_poles_directions)
        self.menuIPF.addAction(self.actiondraw_IPF)
        self.menubar.addAction(self.menuSave.menuAction())
        self.menubar.addAction(self.menuStructure.menuAction())
        self.menubar.addAction(self.menuAngle.menuAction())
        self.menubar.addAction(self.menuSchmid_factor.menuAction())
        self.menubar.addAction(self.menuXyz_directions.menuAction())
        self.menubar.addAction(self.menuList.menuAction())
        self.menubar.addAction(self.menuWidth.menuAction())
        self.menubar.addAction(self.menuIntersections.menuAction())
        self.menubar.addAction(self.menuDiffraction.menuAction())
        self.menubar.addAction(self.menuIPF.menuAction())

        self.retranslateUi(StereoProj)
        QtCore.QMetaObject.connectSlotsByName(StereoProj)
        StereoProj.setTabOrder(self.abc_entry, self.alphabetagamma_entry)
        StereoProj.setTabOrder(self.alphabetagamma_entry, self.e_entry)
        StereoProj.setTabOrder(self.e_entry, self.reciprocal_checkBox)
        StereoProj.setTabOrder(self.reciprocal_checkBox, self.space_group_Box)
        StereoProj.setTabOrder(self.space_group_Box, self.d_Slider)
        StereoProj.setTabOrder(self.d_Slider, self.color_trace_bleu)
        StereoProj.setTabOrder(self.color_trace_bleu, self.color_trace_vert)
        StereoProj.setTabOrder(self.color_trace_vert, self.color_trace_rouge)
        StereoProj.setTabOrder(self.color_trace_rouge, self.style_box)
        StereoProj.setTabOrder(self.style_box, self.wulff_button)
        StereoProj.setTabOrder(self.wulff_button, self.size_var)
        StereoProj.setTabOrder(self.size_var, self.text_size_entry)
        StereoProj.setTabOrder(self.text_size_entry, self.real_space_checkBox)
        StereoProj.setTabOrder(self.real_space_checkBox, self.tilt_angle_entry)
        StereoProj.setTabOrder(self.tilt_angle_entry, self.image_angle_entry)
        StereoProj.setTabOrder(self.image_angle_entry, self.hexa_button)
        StereoProj.setTabOrder(self.hexa_button, self.uvw_button)
        StereoProj.setTabOrder(self.uvw_button, self.reset_view_button)
        StereoProj.setTabOrder(self.reset_view_button, self.diff_entry)
        StereoProj.setTabOrder(self.diff_entry, self.tilt_entry)
        StereoProj.setTabOrder(self.tilt_entry, self.inclinaison_entry)
        StereoProj.setTabOrder(self.inclinaison_entry, self.button_trace)
        StereoProj.setTabOrder(self.button_trace, self.rot_gm_button)
        StereoProj.setTabOrder(self.rot_gm_button, self.rot_g_entry)
        StereoProj.setTabOrder(self.rot_g_entry, self.rot_gp_button)
        StereoProj.setTabOrder(self.rot_gp_button, self.lock_checkButton)
        StereoProj.setTabOrder(self.lock_checkButton, self.angle_alpha_buttonm)
        StereoProj.setTabOrder(self.angle_alpha_buttonm, self.angle_alpha_entry)
        StereoProj.setTabOrder(self.angle_alpha_entry, self.angle_alpha_buttonp)
        StereoProj.setTabOrder(self.angle_alpha_buttonp, self.alpha_signBox)
        StereoProj.setTabOrder(self.alpha_signBox, self.angle_beta_buttonm)
        StereoProj.setTabOrder(self.angle_beta_buttonm, self.angle_beta_entry)
        StereoProj.setTabOrder(self.angle_beta_entry, self.angle_beta_buttonp)
        StereoProj.setTabOrder(self.angle_beta_buttonp, self.beta_signBox)
        StereoProj.setTabOrder(self.beta_signBox, self.angle_z_buttonm)
        StereoProj.setTabOrder(self.angle_z_buttonm, self.angle_z_entry)
        StereoProj.setTabOrder(self.angle_z_entry, self.angle_z_buttonp)
        StereoProj.setTabOrder(self.angle_z_buttonp, self.theta_signBox)
        StereoProj.setTabOrder(self.theta_signBox, self.reset_angle_button)
        StereoProj.setTabOrder(self.reset_angle_button, self.pole_entry)
        StereoProj.setTabOrder(self.pole_entry, self.addpole_button)
        StereoProj.setTabOrder(self.addpole_button, self.undo_addpole_button)
        StereoProj.setTabOrder(self.undo_addpole_button, self.sym_button)
        StereoProj.setTabOrder(self.sym_button, self.undo_sym_button)
        StereoProj.setTabOrder(self.undo_sym_button, self.trace_plan_button)
        StereoProj.setTabOrder(self.trace_plan_button, self.undo_trace_plan_button)
        StereoProj.setTabOrder(self.undo_trace_plan_button, self.trace_plan_sym_button)
        StereoProj.setTabOrder(self.trace_plan_sym_button, self.undo_trace_plan_sym_button)
        StereoProj.setTabOrder(self.undo_trace_plan_sym_button, self.trace_schmid_button)
        StereoProj.setTabOrder(self.trace_schmid_button, self.undo_trace_schmid)
        StereoProj.setTabOrder(self.undo_trace_schmid, self.trace_cone_button)
        StereoProj.setTabOrder(self.trace_cone_button, self.inclination_entry)
        StereoProj.setTabOrder(self.inclination_entry, self.undo_trace_cone_button)
        StereoProj.setTabOrder(self.undo_trace_cone_button, self.norm_button)
        StereoProj.setTabOrder(self.norm_button, self.phi1phiphi2_entry)
        StereoProj.setTabOrder(self.phi1phiphi2_entry, self.button_trace2)

    def retranslateUi(self, StereoProj):
        _translate = QtCore.QCoreApplication.translate
        StereoProj.setWindowTitle(_translate("StereoProj", "Stereo-Proj"))
        StereoProj.setAccessibleName(_translate("StereoProj", "Stereoproj"))
        self.centralwidget.setAccessibleName(_translate("StereoProj", "Stereoproj"))
        self.groupBox_2.setTitle(_translate("StereoProj", "Axis/Rotation"))
        self.rot_gm_button.setText(_translate("StereoProj", "-"))
        self.diff_label.setText(_translate("StereoProj", "g-vector"))
        self.rot_diff_label.setText(_translate("StereoProj", "Rotation along g"))
        self.button_trace.setText(_translate("StereoProj", "PLOT"))
        self.rot_gp_button.setText(_translate("StereoProj", "+"))
        self.tilt_label.setText(_translate("StereoProj", "Tilt α,β,θ"))
        self.inclination_label.setText(_translate("StereoProj", "Inclination"))
        self.groupBox_5.setTitle(_translate("StereoProj", "Euler Angles"))
        self.lab_euler2.setText(_translate("StereoProj", "φ 1 , Φ , φ2"))
        self.button_trace2.setText(_translate("StereoProj", "PLOT"))
        self.lab_coord.setText(_translate("StereoProj", "Tilt, Inclination"))
        self.groupBox_9.setTitle(_translate("StereoProj", "Layout"))
        self.uvw_button.setText(_translate("StereoProj", "uvw"))
        self.hexa_button.setText(_translate("StereoProj", "hexa"))
        self.color_trace_bleu.setText(_translate("StereoProj", "blue"))
        self.color_trace_vert.setText(_translate("StereoProj", "green"))
        self.text_size_label.setText(_translate("StereoProj", "Text size"))
        self.color_trace_rouge.setText(_translate("StereoProj", "red"))
        self.size_var_label.setText(_translate("StereoProj", "Marker size"))
        self.tilt_angle_label.setText(_translate("StereoProj", "diff α-tilt/y angle"))
        self.label_2.setText(_translate("StereoProj", "img α-tilt/y angle"))
        self.reset_view_button.setText(_translate("StereoProj", "Update/Reset view"))
        self.real_space_checkBox.setText(_translate("StereoProj", "img/diff"))
        self.style_box.setText(_translate("StereoProj", "open/filled"))
        self.wulff_button.setText(_translate("StereoProj", "Wulff net"))
        self.groupBox_4.setTitle(_translate("StereoProj", "Pole/Plane"))
        self.trace_plan_sym_button.setText(_translate("StereoProj", "Sym Plane"))
        self.sym_button.setText(_translate("StereoProj", "Symmetry"))
        self.undo_trace_cone_button.setText(_translate("StereoProj", "-"))
        self.trace_plan_button.setText(_translate("StereoProj", " Plane"))
        self.undo_trace_plan_sym_button.setText(_translate("StereoProj", "-"))
        self.trace_cone_button.setText(_translate("StereoProj", "Cone"))
        self.undo_trace_schmid.setText(_translate("StereoProj", "-"))
        self.norm_button.setText(_translate("StereoProj", "dhkl"))
        self.undo_trace_plan_button.setText(_translate("StereoProj", "-"))
        self.undo_addpole_button.setText(_translate("StereoProj", "-"))
        self.addpole_button.setText(_translate("StereoProj", "Add"))
        self.undo_sym_button.setText(_translate("StereoProj", "-"))
        self.trace_schmid_button.setText(_translate("StereoProj", "Schmid"))
        self.groupBox_3.setTitle(_translate("StereoProj", "Rotation"))
        self.angle_beta_label.setText(_translate("StereoProj", "<html><head/><body><p>β</p></body></html>"))
        self.angle_z_buttonm.setText(_translate("StereoProj", "-"))
        self.angle_beta_buttonp.setText(_translate("StereoProj", "+"))
        self.label.setText(_translate("StereoProj", "AC"))
        self.angle_beta_buttonm.setText(_translate("StereoProj", "-"))
        self.angle_z_label.setText(_translate("StereoProj", "<html><head/><body><p>θ</p></body></html>"))
        self.lock_checkButton.setText(_translate("StereoProj", "Lock Axes"))
        self.angle_alpha_buttonm.setText(_translate("StereoProj", "-"))
        self.angle_z_buttonp.setText(_translate("StereoProj", "+"))
        self.angle_alpha_buttonp.setText(_translate("StereoProj", "+"))
        self.angle_alpha_label.setText(_translate("StereoProj", "<html><head/><body><p>α</p></body></html>"))
        self.reset_angle_button.setText(_translate("StereoProj", "Reset"))
        self.groupBox.setTitle(_translate("StereoProj", "Crystal Parameters"))
        self.alphabetagamma_label.setText(_translate("StereoProj", "<p>&alpha;, &beta;, &gamma;</p>"))
        self.abc_label.setText(_translate("StereoProj", "a,b,c"))
        self.e_label.setText(_translate("StereoProj", "max indices"))
        self.d_label.setText(_translate("StereoProj", "d"))
        self.reciprocal_checkBox.setText(_translate("StereoProj", "Reciprocal indices"))
        self.menuSave.setTitle(_translate("StereoProj", "Save"))
        self.menuStructure.setTitle(_translate("StereoProj", "Structure"))
        self.menuAngle.setTitle(_translate("StereoProj", "Angle"))
        self.menuSchmid_factor.setTitle(_translate("StereoProj", "Schmid factor"))
        self.menuXyz_directions.setTitle(_translate("StereoProj", "xyz directions"))
        self.menuWidth.setTitle(_translate("StereoProj", "Width"))
        self.menuIntersections.setTitle(_translate("StereoProj", "intersections"))
        self.menuDiffraction.setTitle(_translate("StereoProj", "diffraction"))
        self.menuList.setTitle(_translate("StereoProj", "list"))
        self.menuIPF.setTitle(_translate("StereoProj", "IPF"))
        self.actionSave_figure.setText(_translate("StereoProj", "Save figure"))
        self.actionCalculate_Schmid_factor.setText(_translate("StereoProj", "calculate Schmid factor"))
        self.actionCalculate_angle.setText(_translate("StereoProj", "Calculate angle"))
        self.actionCalculate_xyz.setText(_translate("StereoProj", "calculate xyz directions"))
        self.actionCalculate_apparent_width.setText(_translate("StereoProj", "Calculate apparent width"))
        self.actionPlanes.setText(_translate("StereoProj", "planes"))
        self.actionProj_directions.setText(_translate("StereoProj", "proj. directions"))
        self.actionPlane_cone.setText(_translate("StereoProj", "plane-cone"))
        self.actionCalculate_intersections.setText(_translate("StereoProj", "Calculate intersections"))
        self.actionHkl_uvw.setText(_translate("StereoProj", "hkl <> uvw"))
        self.actionPlot_Kikuchi_lines.setText(_translate("StereoProj", "plot Kikuchi lines or diffraction pattern"))
        self.actionShow_list_of_poles_directions.setText(_translate("StereoProj", "show list of poles/directions"))
        self.actiondraw_IPF.setText(_translate("StereoProj", "draw IPF"))



from __future__ import division
import numpy as np
from PyQt5 import QtWidgets
import sys
import os
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from matplotlib import pyplot as plt
from . import crystalUI


def Rot(th, a, b, c):

    aa = a / np.linalg.norm([a, b, c])
    bb = b / np.linalg.norm([a, b, c])
    cc = c / np.linalg.norm([a, b, c])
    c1 = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], float)
    c2 = np.array([[aa**2, aa * bb, aa * cc], [bb * aa, bb**2, bb * cc], [cc * aa,
                                                                          cc * bb, cc**2]], float)
    c3 = np.array([[0, -cc, bb], [cc, 0, -aa], [-bb, aa, 0]], float)
    R = np.cos(th) * c1 + (1 - np.cos(th)) * c2 + np.sin(th) * c3

    return R


def coord_ortho(P, plan):
    global Dstar, varname
    ref = np.array([0, 0, 1])
    planN = np.dot(Dstar, plan)
    the = np.arccos(np.dot(planN, ref) / (np.linalg.norm(ref) * np.linalg.norm(planN)))
    if plan[0] == 0 and plan[1] == 0:
        axe = ref
    else:
        axe = np.cross(planN, ref)

    M = np.dot(Rot(the, axe[0], axe[1], axe[2]), P)
    return M


def unique(a):
    a = np.sort(a)
    b = np.diff(a)
    b = np.r_[1, b]
    return a[b != 0]


def calcul():
    global vec, varname, atom0, Dstar, taille, zoom, EL, Dz, dsc_cond

    if varname != 0:
        f_space = open(varname[0], "r")
    else:
        varname = getFileName()
        f_space = open(varname[0], "r")

    crist = []

    for line in f_space:
        crist.append(list(map(str, line.split())))

    f_space.close()
    vec = []
    atom0 = []
    for i in range(0, np.size(crist)):
        if np.size(crist[i]) == 3:
            vec.append(crist[i])
        else:
            atom0.append(crist[i])

    vec = np.array(vec, float)
    E = np.array([0, 0, 0, 0])
    EL = np.array([0, 0, 0, 0, 0])
    H = np.array([[0]])
    Dz = np.array([0, 0, 0])

    if ui.dsc_box.isChecked() is False:
        atom0 = np.array(atom0, float)
        maxi = int(atom0[np.shape(atom0)[0] - 1, 0])
        for h in range(1, maxi + 1):
            Ca = calcul_atom(atom0[atom0[:, 0] == h])

            E = np.vstack((E, Ca[0]))
            Dz = np.vstack((Dz, Ca[1]))
            H = np.vstack((H, h * np.ones((np.shape(Ca[0])[0], 1))))

    if ui.dsc_box.isChecked():
        atom0 = np.array([1, 0, 0, 0])
        h = 1
        Ca = calcul_atom(atom0)
        E = np.vstack((E, Ca[0]))
        Dz = np.vstack((Dz, Ca[1]))
        H = np.vstack((H, h * np.ones((np.shape(Ca[0])[0], 1))))

    EL = np.append(E, H, axis=1)
    EL = np.delete(EL, (0), axis=0)
    Dz = np.delete(Dz, (0), axis=0)


def trace():
    global vec, varname, atom0, Dstar, taille, zoom, EL, Dz, fi

    fi.clear()

    sim = int(ui.markers_entry.text())
    if ui.dsc_box.isChecked() is False:
        if ui.square_box.isChecked() is False:
            fi.scatter(EL[:, 0], EL[:, 1], s=sim, c=EL[:, 3], marker='o')
        else:
            fi.scatter(EL[:, 0], EL[:, 1], s=sim, c=EL[:, 3], marker='s')

        if ui.atoms_box.isChecked():
            for k in range(0, np.shape(EL)[0]):
                fi.annotate(str(int(EL[k, 4])), (EL[k, 0], EL[k, 1]))

        if ui.labels_box.isChecked():
            for q in range(0, np.shape(EL)[0]):
                at = Dz[q, :]
                at = np.dot(at, Dstar)

                vector = str(np.around(at[0], decimals=3)) + ',' + str(np.around(at[1], decimals=3)) + ',' + str(np.around(at[2], decimals=3))
                fi.annotate(vector, (EL[q, 0], EL[q, 1]))

    if ui.dsc_box.isChecked():
        theta = np.float64(ui.angle_entry.text())
        theta = theta * np.pi / 180
        ELr = np.dot(EL[:, 0:3], Rot(theta, 0, 0, 1))
        M = unique(EL[:, 3])

        Ma0 = []
        Ma1 = []
        Ma2 = []
        Ma = []
        m = ('o', 's', '^', '*', 'h')
        abc = ui.abc_entry.text().split(",")
        a = np.float64(abc[0])
        b = np.float64(abc[1])
        c = np.float64(abc[2])
        ee = np.float64(ui.precision_entry.text())
        if ui.coincidence_box.isChecked():
            coi = np.array([0])
            ep = 0.01 * ee * np.max([a, b, c])
            for i in range(0, np.shape(EL[:, 3])[0]):
                for j in range(0, np.shape(ELr[:, 0])[0]):
                    if np.linalg.norm(EL[i, 0:3] - ELr[j, :]) < ep:
                        coi = np.hstack((coi, i))
            coi = np.unique(coi)
        for t in range(0, np.shape(M)[0]):
            for i in range(0, np.shape(EL[:, 3])[0]):
                if EL[i, 3] == M[t]:

                    if t > np.size(m) - 1:
                        Ma0.append('o')
                        Ma1.append('o')

                    else:
                        Ma0.append(m[t])
                        Ma1.append(m[t])
                    if ui.layers_box.isChecked():
                        Ma2.append(str(t))
        else:
            Ma.append('D')

        for y in range(0, np.shape(EL[:, 3])[0]):
            fi.scatter(ELr[y, 0], ELr[y, 1], s=sim, marker=Ma0[y], color='white', edgecolor='black')
            fi.scatter(EL[y, 0], EL[y, 1], s=sim, marker=Ma1[y], color='black', edgecolor='black')
            if ui.layers_box.isChecked():
                fi.text(EL[y, 0], EL[y, 1], Ma2[y])
                fi.text(ELr[y, 0], ELr[y, 1], Ma2[y])
        if ui.coincidence_box.isChecked():
            for z in coi:
                fi.scatter(EL[z, 0], EL[z, 1], s=sim * 1.5, marker=Ma0[z], color='blue', edgecolor='black')

        if ui.labels_box.isChecked():
            for q in range(0, np.shape(EL)[0]):
                at = Dz[q, :]
                at = np.dot(at, Dstar)

                vector = str(np.around(at[0], decimals=1)) + ',' + str(np.around(at[1], decimals=1)) + ',' + str(np.around(at[2], decimals=1))
                fi.annotate(vector, (EL[q, 0], EL[q, 1]))
    fi.axis('off')
    fi.axis('equal')
    fi.figure.canvas.draw()


def rep():
    global varname, vec, E, C, Dz, atom0

    fi.clear()

    sim = int(ui.markers_entry.text())
    if varname != 0:
        f_space = open(varname, "r")
    else:
        varname = getFileName()
        f_space = open(varname, "r")

    crist = []

    for line in f_space:
        crist.append(list(map(str, line.split())))

    f_space.close()
    vec = []
    atom0 = []
    for i in range(0, np.size(crist)):
        if np.size(crist[i]) == 3:
            vec.append(crist[i])
        else:
            atom0.append(crist[i])

    vec = np.array(vec, float)
    atom0 = np.array(atom0, float)

    maxi = int(atom0[np.shape(atom0)[0] - 1, 0])

    for h in range(1, maxi + 1):

        E = calcul_rep(atom0[atom0[:, 0] == h])

        fi.scatter(E[:, 0], E[:, 1], E[:, 2], s=sim, c=str(h / maxi))

    fi.axis('off')
    fi.figure.canvas.draw()


def calcul_rep(atom):
    global Dstar, varname, C, D0, Dz, planN, plan, vec, c
    abc = ui.abc_entry.text().split(",")
    a = np.float64(abc[0])
    b = np.float64(abc[1])
    c = np.float64(abc[2])
    alphabetagamma = ui.alphabetagamma_entry.text().split(",")
    alpha = np.float64(alphabetagamma[0])
    beta = np.float64(alphabetagamma[1])
    gamma = np.float64(alphabetagamma[2])
    alp = alpha * np.pi / 180
    bet = beta * np.pi / 180
    gam = gamma * np.pi / 180

    V = a * b * c * np.sqrt(1 - (np.cos(alp)**2) - (np.cos(bet))**2 - (np.cos(gam))**2 + 2 * b * c * np.cos(alp) * np.cos(bet) * np.cos(gam))
    D = np.array([[a, b * np.cos(gam), c * np.cos(bet)], [0, b * np.sin(gam), c * (np.cos(alp) - np.cos(bet) * np.cos(gam)) / np.sin(gam)], [0, 0, V / (a * b * np.sin(gam))]])
    Dstar = np.transpose(np.linalg.inv(D))

    n = ui.size_entry.text().split(",")
    na_rep = int(n[0])
    nb_rep = int(n[1])
    nc_rep = int(n[2])

    A = np.zeros((np.shape(atom)[0], np.shape(atom)[1] - 1))
    w = 0

    for v in range(0, np.shape(atom)[0]):

        A[w, :] = np.dot(D, np.array([atom[v, 1], atom[v, 2], atom[v, 3]]))
        w = w + 1

    atom_pos = np.array(A[0, :])
    for f in range(0, np.shape(A)[0]):
        for i in range(-na_rep, na_rep + 1):
            for j in range(-nb_rep, nb_rep + 1):
                for k in range(-nc_rep, nc_rep + 1):

                    atom_pos = np.vstack((atom_pos, A[f, :] + i * a * vec[0, :] + j * b * vec[1, :] + k * c * vec[2, :]))

    return atom_pos


def calcul_atom(atom):
    global Dstar, varname, C, D0, planN, plan, vec, atom_pos

    abc = ui.abc_entry.text().split(",")
    a = np.float64(abc[0])
    b = np.float64(abc[1])
    c = np.float64(abc[2])
    alphabetagamma = ui.alphabetagamma_entry.text().split(",")
    alpha = np.float64(alphabetagamma[0])
    beta = np.float64(alphabetagamma[1])
    gamma = np.float64(alphabetagamma[2])
    alp = alpha * np.pi / 180
    bet = beta * np.pi / 180
    gam = gamma * np.pi / 180

    V = a * b * c * np.sqrt(1 - (np.cos(alp)**2) - (np.cos(bet))**2 - (np.cos(gam))**2 + 2 * b * c * np.cos(alp) * np.cos(bet) * np.cos(gam))
    D = np.array([[a, b * np.cos(gam), c * np.cos(bet)], [0, b * np.sin(gam), c * (np.cos(alp) - np.cos(bet) * np.cos(gam)) / np.sin(gam)], [0, 0, V / (a * b * np.sin(gam))]])
    Dstar = np.transpose(np.linalg.inv(D))

    n = ui.size_entry.text().split(",")
    na = int(n[0])
    nb = int(n[1])
    nc = int(n[2])

    if ui.dsc_box.isChecked() is False:
        A = np.zeros((np.shape(atom)[0], np.shape(atom)[1] - 1))
        w = 0
        for v in range(0, np.shape(atom)[0]):
            A[w, :] = np.dot(D, np.array([atom[v, 1], atom[v, 2], atom[v, 3]]))
            w = w + 1
        atom_pos = np.array(A[0, :])
        for f in range(0, np.shape(A)[0]):
            for i in range(-na, na + 1):
                for j in range(-nb, nb + 1):
                    for k in range(-nc, nc + 1):
                        atom_pos = np.vstack((atom_pos, A[f, :] + i * a * vec[0, :] + j * b * vec[1, :] + k * c * vec[2, :]))

    if ui.dsc_box.isChecked():
        atom_pos = np.array([0, 0, 0])
        for i in range(-na, na + 1):
            for j in range(-nb, nb + 1):
                for k in range(-nc, nc + 1):
                    atom_pos = np.vstack((atom_pos, i * a * vec[0, :] + j * b * vec[1, :] + k * c * vec[2, :]))
    pl = ui.plane_entry.text().split(",")
    h = np.float64(pl[0])
    k = np.float64(pl[1])
    l = np.float64(pl[2])

    plan = np.array([h, k, l])
    planN = np.dot(Dstar, plan)
    Dz = np.array([0, 0, 0])
    D0 = np.array([0])
    C = np.array([0, 0, 0])

    L = np.zeros(np.shape(atom_pos)[0])
    tt = 0
    for t in range(0, np.shape(atom_pos)[0]):
        L[tt] = np.around(np.dot(planN, atom_pos[t]), decimals=4)
        tt = tt + 1

    Le = unique(np.abs(L))
    lay = ui.layers_entry.text().split(",")
    cc = int(lay[0])
    dd = int(lay[1])

    for y in range(cc, dd):
        for i in range(0, np.shape(atom_pos)[0]):

            if (np.around(np.dot(planN, atom_pos[i]), decimals=4)) == Le[y]:

                Dz = np.vstack((Dz, atom_pos[i]))

    for j in range(1, np.shape(Dz)[0]):
        C = np.vstack((C, coord_ortho(Dz[j, :], plan)))
        D0 = np.vstack((D0, 1 + np.abs(np.around(np.dot(planN, Dz[j]), decimals=4))))
    C = np.delete(C, (0), axis=0)
    D0 = np.delete(D0, (0), axis=0)
    Dz = np.delete(Dz, (0), axis=0)
    F = np.append(C, D0, axis=1)

    return F, Dz, atom_pos


def getFileName():
    global varname
    varname = QtWidgets.QFileDialog.getOpenFileName(None, 'OpenFile')
    return varname

#######################################
#
# Save image as png
#
#########################################


def image_save():
    filename = QtWidgets.QFileDialog.getSaveFileName(Index, "Save file", "", ".png")
    f = str(filename[0]) + ".png"
    canvas.print_figure(f)


##################################################
#
# Add matplotlib toolbar to zoom and pan
#
###################################################


class NavigationToolbar(NavigationToolbar):
    # only display the buttons we need
    toolitems = [t for t in NavigationToolbar.toolitems if
                 t[0] in ('Pan', 'Zoom')]

    def set_message(self, msg):
        pass
###########################################################
#
# Structure
#
##############################################################


def structure(item):
    global x0, var_hexa, d_label_var, e_entry

    ui.abc_entry.setText(str(item[1]) + ',' + str(item[2]) + ',' + str(item[3]))
    ui.alphabetagamma_entry.setText(str(item[4]) + ',' + str(item[5]) + ',' + str(item[6]))


def except_hook(cls, exception, traceback):
    sys.__excepthook__(cls, exception, traceback)


if __name__ == "__main__":

    app = QtWidgets.QApplication(sys.argv)
    sys.excepthook = except_hook
    QtWidgets.qApp.setApplicationName("Crystal")
    Index = QtWidgets.QMainWindow()
    ui = crystalUI.Ui_Crystal()
    ui.setupUi(Index)
    figure = plt.figure()
    fi = figure.add_subplot(111)
    fi.axis("off")
    canvas = FigureCanvas(figure)
    ui.mplvl.addWidget(canvas)
    toolbar = NavigationToolbar(canvas, canvas)
    toolbar.setMinimumWidth(601)


# Read structure file

    file_struct = open(os.path.join(os.path.dirname(__file__), 'structure.txt'), "r")
    x0 = []
    for line in file_struct:
        x0.append(list(map(str, line.split())))
    i = 0
    file_struct.close()
    for item in x0:
        entry = ui.menuStructure.addAction(item[0])
        entry.triggered.connect(lambda checked, item=item: structure(item))
        i = i + 1

    ui.actionSave_figure.triggered.connect(image_save)
    varname = 0
    ui.calculate_button.clicked.connect(calcul)
    ui.draw_button.clicked.connect(trace)
    ui.changeStruct_button.clicked.connect(getFileName)
    ui.abc_entry.setText('1,1,1')
    ui.alphabetagamma_entry.setText('90,90,90')
    ui.plane_entry.setText('1,1,1')
    ui.size_entry.setText('2,2,2')
    ui.layers_entry.setText('0,2')
    ui.markers_entry.setText('30')
    ui.precision_entry.setText('5')
    Index.show()
    sys.exit(app.exec_())



from __future__ import division
import numpy as np
from PyQt5 import QtGui, QtCore, QtWidgets
import sys
import os
import functools
from PIL import Image
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from matplotlib import pyplot as plt
import matplotlib as mpl
from . import misorientationUI


################
#       Misc
################

def unique_rows(a):
    a = np.ascontiguousarray(a)
    unique_a = np.unique(a.view([('', a.dtype)] * a.shape[1]))
    return unique_a.view(a.dtype).reshape((unique_a.shape[0], a.shape[1]))


def GCD(a, b, rtol=1e-05, atol=1e-08):
    t = min(abs(a), abs(b))
    while abs(b) > rtol * t + atol:
        a, b = b, a % b
    return a


# "
# Projection
####################################################################

def proj(x, y, z):

    if z == 1:
        X = 0
        Y = 0
    elif z < -0.000001:
        X = 'nan'
        Y = 'nan'
    else:

        X = x / (1 + z)
        Y = y / (1 + z)

    return np.array([X, Y], float)


def proj2(x, y, z):

    if z == 1:
        X = 0
        Y = 0
    elif z < -0.000001:
        X = -x / (1 - z)
        Y = -y / (1 - z)
    else:

        X = x / (1 + z)
        Y = y / (1 + z)

    return np.array([X, Y, z], float)


def proj_gnomonic(x, y, z):

    if z == 0:
        X = x
        Y = y

    else:

        X = x / z
        Y = y / z

    return np.array([X, Y], float)

# def proj_ortho(x,y,z):
#
#    return np.array([x,y],float)


###################################################################
# Rotation Euler
#
##################################################################

def rotation(phi1, phi, phi2):
    phi1 = phi1 * np.pi / 180
    phi = phi * np.pi / 180
    phi2 = phi2 * np.pi / 180
    R = np.array([[np.cos(phi1) * np.cos(phi2) - np.cos(phi) * np.sin(phi1) * np.sin(phi2),
                   -np.cos(phi) * np.cos(phi2) * np.sin(phi1) - np.cos(phi1) *
                   np.sin(phi2), np.sin(phi) * np.sin(phi1)], [np.cos(phi2) * np.sin(phi1)
                                                               + np.cos(phi) * np.cos(phi1) * np.sin(phi2), np.cos(phi) * np.cos(phi1)
                                                               * np.cos(phi2) - np.sin(phi1) * np.sin(phi2), -np.cos(phi1) * np.sin(phi)],
                  [np.sin(phi) * np.sin(phi2), np.cos(phi2) * np.sin(phi), np.cos(phi)]], float)
    return R

###################################################################

# Rotation around a given axis
#
##################################################################


def Rot(th, a, b, c):
    th = th * np.pi / 180
    no = np.linalg.norm([a, b, c])
    aa = a / no
    bb = b / no
    cc = c / no
    c1 = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1]], float)
    c2 = np.array([[aa**2, aa * bb, aa * cc], [bb * aa, bb**2, bb * cc], [cc * aa,
                                                                          cc * bb, cc**2]], float)
    c3 = np.array([[0, -cc, bb], [cc, 0, -aa], [-bb, aa, 0]], float)
    R = np.cos(th) * c1 + (1 - np.cos(th)) * c2 + np.sin(th) * c3

    return R

#######################
#
# Layout functions
#
#######################


def color_trace(i_c):

    if i_c == 2:
        if ui.color_trace_bleu_2.isChecked():
            color_trace = 1
        if ui.color_trace_vert_2.isChecked():
            color_trace = 2
        if ui.color_trace_rouge_2.isChecked():
            color_trace = 3
    else:
        if ui.color_trace_bleu.isChecked():
            color_trace = 1
        if ui.color_trace_vert.isChecked():
            color_trace = 2
        if ui.color_trace_rouge.isChecked():
            color_trace = 3
    return color_trace


def var_uvw():
    var_uvw = 0
    if ui.uvw_button.isChecked():
        var_uvw = 1

    return var_uvw


def var_hexa(i_c):
    if i_c == 2:
        var_hexa = 0
        if ui.hexa_button_2.isChecked():
            var_hexa = 1
    else:
        var_hexa = 0
        if ui.hexa_button.isChecked():
            var_hexa = 1
    return var_hexa


def var_carre(i_c):
    if i_c == 2:
        var_carre = 0
        if ui.style_box.isChecked():
            var_carre = 1
    else:
        var_carre = 0
        if ui.style_box_2.isChecked():
            var_carre = 1

    return var_carre

####################################################################
#
#  Crystal definition
#
##################################################################


def crist():
    global i_c, axes, axesh, D, Dstar, V, G, hexa_cryst1, hexa_cryst2, e, e2

    abc2 = ui.abc_entry_2.text().split(",")
    alphabetagamma2 = ui.alphabetagamma_entry_2.text().split(",")
    e2 = np.int32(ui.e_entry_2.text())
    abc = ui.abc_entry.text().split(",")
    alphabetagamma = ui.alphabetagamma_entry.text().split(",")
    e = np.int32(ui.e_entry.text())
    hexa_cryst1 = 0

    if np.float64(alphabetagamma[0]) == 90 and np.float64(alphabetagamma[1]) == 90 and np.float64(alphabetagamma[2]) == 120:
        hexa_cryst1 = 1
        dmip1 = np.float64(abc[0]) / 2 - 0.0001e-10
    else:
        dmip1 = 0

    hexa_cryst2 = 0
    if np.float64(alphabetagamma2[0]) == 90 and np.float64(alphabetagamma2[1]) == 90 and np.float64(alphabetagamma2[2]) == 120:
        hexa_cryst2 = 1
        dmip2 = np.float64(abc2[0]) / 2 - 0.0001e-10
    else:
        dmip2 = 0

    ui.d1_Slider.setMinimum(0)
    ui.d1_Slider.setMaximum(int(np.amax([np.float64(abc[0]), np.float64(abc[1]), np.float64(abc[2])]) * 100))
    ui.d1_Slider.setSingleStep(100)
    ui.d1_Slider.setValue(dmip1 * 100)
    ui.d_label_var.setText(str(np.around(dmip1, decimals=3)))

    ui.d2_Slider.setMinimum(0)
    ui.d2_Slider.setMaximum(int(np.amax([np.float64(abc2[0]), np.float64(abc2[1]), np.float64(abc2[2])]) * 100))
    ui.d2_Slider.setSingleStep(100)
    ui.d2_Slider.setValue(int(dmip2 * 100))

    ui.d_label_var_2.setText(str(np.around(dmip2, decimals=3)))

    axes1, axesh1, D1, Dstar1, V1, G1 = crist_axes(abc, alphabetagamma, e, dmip1, hexa_cryst1, 1)
    axes2, axesh2, D2, Dstar2, V2, G2 = crist_axes(abc2, alphabetagamma2, e2, dmip2, hexa_cryst2, 2)
    if ui.crystal1_radioButton.isChecked():
        D, Dstar, V, G = D1, Dstar1, V1, G1
    if ui.crystal2_radioButton.isChecked():
        D, Dstar, V, G = D2, Dstar2, V2, G2
    axes = np.vstack((axes1, axes2))
    axesh = np.vstack((axesh1, axesh2))


def crist_mat(abc, alphabetagamma):
    a = np.float64(abc[0]) * 1e-10
    b = np.float64(abc[1]) * 1e-10
    c = np.float64(abc[2]) * 1e-10
    alpha = np.float64(alphabetagamma[0])
    beta = np.float64(alphabetagamma[1])
    gamma = np.float64(alphabetagamma[2])
    alpha = alpha * np.pi / 180
    beta = beta * np.pi / 180
    gamma = gamma * np.pi / 180
    V = a * b * c * np.sqrt(1 - (np.cos(alpha)**2) - (np.cos(beta))**2 - (np.cos(gamma))**2 + 2 * np.cos(alpha) * np.cos(beta) * np.cos(gamma))
    D = np.array([[a, b * np.cos(gamma), c * np.cos(beta)], [0, b * np.sin(gamma), c * (np.cos(alpha) - np.cos(beta) * np.cos(gamma)) / np.sin(gamma)], [0, 0, V / (a * b * np.sin(gamma))]])
    Dstar = np.transpose(np.linalg.inv(D))
    G = np.array([[a**2, a * b * np.cos(gamma), a * c * np.cos(beta)], [a * b * np.cos(gamma), b**2, b * c * np.cos(alpha)], [a * c * np.cos(beta), b * c * np.cos(alpha), c**2]])
    return D, Dstar, V, G


def crist_axes(abc, alphabetagamma, e, dmip, hexa_cryst, i_c):

    D, Dstar, V, G = crist_mat(abc, alphabetagamma)
    axes = np.zeros(((2 * e + 1)**3 - 1, 4))
    axesh = np.zeros(((2 * e + 1)**3 - 1, 10))
    axesh[:, 4] = color_trace(i_c)
    axesh[:, 8] = var_hexa(i_c)
    axesh[:, 9] = var_carre(i_c)
    axesh[:, 7] = i_c
    axes[:, 3] = i_c

    id = 0
    for i in range(-e, e + 1):
        for j in range(-e, e + 1):
            for k in range(-e, e + 1):
                if (i, j, k) != (0, 0, 0):
                    if var_uvw() == 0:
                        Ma = np.dot(Dstar, np.array([i, j, k], float))
                        axesh[id, 3] = 0
                    else:
                        Ma = np.dot(D, np.array([i, j, k], float))
                        axesh[id, 3] = 1

                    m = np.abs(functools.reduce(lambda x, y: GCD(x, y), [i, j, k]))
                    if (np.around(i / m) == i / m) & (np.around(j / m) == j / m) & (np.around(k / m) == k / m):
                        axes[id, 0:3] = np.array([i, j, k]) / m
                    else:
                        axes[id, 0:3] = np.array([i, j, k])
                    axesh[id, 0:3] = Ma / np.linalg.norm(Ma)
                    axesh[id, 5] = 1
                    axesh[id, 6] = 1
                    id = id + 1

    axesh = axesh[~np.all(axesh[:, 0:3] == 0, axis=1)]
    axes = axes[~np.all(axes == 0, axis=1)]

    for i in range(0, np.shape(axes)[0]):
        axesh[i, 6] = 1
        d = 1 / (np.sqrt(np.dot(axes[i, 0:3], np.dot(np.linalg.inv(G), axes[i, 0:3])))) * 1e10
        if d < dmip:
            axesh[i, 6] = 0
        if (hexa_cryst == 1 and np.abs(axes[i, 0] + axes[i, 1]) > e):
            axesh[i, 6] = 0

    return axes, axesh, D, Dstar, V, G


######################################################
#
# Reduce number of poles/directions as a function of d-spacing (plus or minus)
#
#######################################################
def dist_restrict1():
    global G, axes, axesh, hexa_cryst1, e
    abc = ui.abc_entry.text().split(",")
    alphabetagamma = ui.alphabetagamma_entry.text().split(",")
    d2 = ui.d1_Slider.value() / 100 * 1e-10
    D, Dstar, V, G = crist_mat(abc, alphabetagamma)

    for i in range(0, np.shape(axes)[0]):
        if axes[i, 3] == 1:
            axesh[i, 6] = 1
            d = 1 / (np.sqrt(np.dot(axes[i, 0:3], np.dot(np.linalg.inv(G), axes[i, 0:3]))))
            if d < d2:
                axesh[i, 6] = 0
            if (hexa_cryst1 == 1 and np.abs(axes[i, 0] + axes[i, 1]) > e):
                axesh[i, 6] = 0

    ui.d_label_var.setText(str(np.around(d2 * 1e10, decimals=3)))
    trace()


def dist_restrict2():
    global G, axes, axesh, hexa_cryst2, e2
    abc = ui.abc_entry_2.text().split(",")
    alphabetagamma = ui.alphabetagamma_entry_2.text().split(",")
    d2 = ui.d2_Slider.value() / 100 * 1e-10

    D, Dstar, V, G = crist_mat(abc, alphabetagamma)

    for i in range(0, np.shape(axes)[0]):
        if axes[i, 3] == 2:
            axesh[i, 6] = 1
            d = 1 / (np.sqrt(np.dot(axes[i, 0:3], np.dot(np.linalg.inv(G), axes[i, 0:3]))))
            if d < d2:
                axesh[i, 6] = 0
            if (hexa_cryst2 == 1 and np.abs(axes[i, 0] + axes[i, 1]) > e2):
                axesh[i, 6] = 0

    ui.d_label_var_2.setText(str(np.around(d2 * 1e10, decimals=3)))
    trace()

###########################################################################
#
# Rotation of the sample. If Lock Axes is off rotation are along y,x,z directions. If not, the y and z axes
# of the sample are locked to the crystal axes when the check box is ticked.
# It mimics double-tilt holder (rotation of alpha along fixed x and rotation of beta along the beta tilt moving axis)
# or  tilt-rotation holder  (rotation of alpha along fixed # x and rotation of z along the z-rotation moving axis).
#
##########################################################################


def euler_label():
    global M, M2
    if np.abs(M[2, 2] - 1) < 0.0001:
        phir = 0
        phi1r = 0
        phi2r = np.arctan2(M[1, 0], M[0, 0]) * 180 / np.pi
    else:
        phir = np.arccos(M[2, 2]) * 180 / np.pi
        phi2r = np.arctan2(M[2, 0], M[2, 1]) * 180 / np.pi
        phi1r = np.arctan2(M[0, 2], -M[1, 2]) * 180 / np.pi

    if np.abs(M2[2, 2] - 1) < 0.0001:
        phir_2 = 0
        phi1r_2 = 0
        phi2r_2 = np.arctan2(M2[1, 0], M2[0, 0]) * 180 / np.pi
    else:
        phir_2 = np.arccos(M2[2, 2]) * 180 / np.pi
        phi2r_2 = np.arctan2(M2[2, 0], M2[2, 1]) * 180 / np.pi
        phi1r_2 = np.arctan2(M2[0, 2], -M2[1, 2]) * 180 / np.pi

    t = str(np.around(phi1r, decimals=1)) + str(',') + str(np.around(phir, decimals=1)) + str(',') + str(np.around(phi2r, decimals=1))
    t2 = str(np.around(phi1r_2, decimals=1)) + str(',') + str(np.around(phir_2, decimals=1)) + str(',') + str(np.around(phi2r_2, decimals=1))

    ui.angle_euler_label.setText(t)
    ui.angle_euler_label_2.setText(t2)


def lock():
    global M, var_lock, M_lock
    i_c = 1
    if ui.crystal1_radioButton.isChecked():
        i_c = 2
    if ui.lock_checkButton.isChecked():
        var_lock = 1
        if i_c == 1:
            M_lock = M
        else:
            M_lock = M2
    else:
        var_lock, M_lock = 0, 0

    return var_lock, M_lock


def rot_alpha_p():
    global angle_alpha, M, M2, a, trP, trC, s_a
    t_ang = -ang_work_space()
    i_c = crystal_check()
    tha = s_a * np.float64(ui.angle_alpha_entry.text())
    t_a_y = np.dot(Rot(t_ang, 0, 0, 1), np.array([0, 1, 0]))
    if i_c == 1:
        M = np.dot(Rot(tha, t_a_y[0], t_a_y[1], t_a_y[2]), M)
    else:
        M2 = np.dot(Rot(tha, t_a_y[0], t_a_y[1], t_a_y[2]), M2)
    trace()
    euler_label()

    angle_alpha = angle_alpha + np.float64(ui.angle_alpha_entry.text())
    ui.angle_alpha_label_2.setText(str(angle_alpha))


def rot_alpha_m():
    global angle_alpha, M, M2, a, trP, trC, s_a
    t_ang = -ang_work_space()
    i_c = crystal_check()
    tha = -s_a * np.float64(ui.angle_alpha_entry.text())
    t_a_y = np.dot(Rot(t_ang, 0, 0, 1), np.array([0, 1, 0]))
    if i_c == 1:
        M = np.dot(Rot(tha, t_a_y[0], t_a_y[1], t_a_y[2]), M)
    else:
        M2 = np.dot(Rot(tha, t_a_y[0], t_a_y[1], t_a_y[2]), M2)
    trace()
    euler_label()
    angle_alpha = angle_alpha - np.float64(ui.angle_alpha_entry.text())
    ui.angle_alpha_label_2.setText(str(angle_alpha))


def rot_beta_m():
    global angle_beta, M, M2, angle_alpha, angle_z, var_lock, M_lock, s_b
    t_ang = -ang_work_space()
    i_c = crystal_check()
    t_a_x = np.dot(Rot(t_ang, 0, 0, 1), np.array([1, 0, 0]))

    if var_lock == 0:
        AxeY = t_a_x
    else:
        A = np.dot(np.linalg.inv(M_lock), t_a_x)
        if i_c == 1:
            AxeY = np.dot(M, A)
        else:
            AxeY = np.dot(M2, A)

    thb = -s_b * np.float64(ui.angle_beta_entry.text())
    if i_c == 1:
        M = np.dot(Rot(thb, AxeY[0], AxeY[1], AxeY[2]), M)
    else:
        M2 = np.dot(Rot(thb, AxeY[0], AxeY[1], AxeY[2]), M2)
    trace()
    euler_label()
    angle_beta = angle_beta - np.float64(ui.angle_beta_entry.text())
    ui.angle_beta_label_2.setText(str(angle_beta))


def rot_beta_p():
    global angle_beta, M, M2, angle_alpha, angle_z, var_lock, M_lock, s_b
    t_ang = -ang_work_space()
    i_c = crystal_check()
    t_a_x = np.dot(Rot(t_ang, 0, 0, 1), np.array([1, 0, 0]))
    if var_lock == 0:
        AxeY = t_a_x
    else:
        A = np.dot(np.linalg.inv(M_lock), t_a_x)
        if i_c == 1:
            AxeY = np.dot(M, A)
        else:
            AxeY = np.dot(M2, A)

    thb = s_b * np.float64(ui.angle_beta_entry.text())
    if i_c == 1:
        M = np.dot(Rot(thb, AxeY[0], AxeY[1], AxeY[2]), M)
    else:
        M2 = np.dot(Rot(thb, AxeY[0], AxeY[1], AxeY[2]), M2)
    trace()
    euler_label()
    angle_beta = angle_beta + np.float64(ui.angle_beta_entry.text())
    ui.angle_beta_label_2.setText(str(angle_beta))


def rot_z_m():
    global angle_beta, M, M2, angle_alpha, angle_z, var_lock, M_lock, s_z, t_ang

    i_c = crystal_check()

    if var_lock == 0:
        AxeZ = np.array([0, 0, 1])
    else:
        A = np.dot(np.linalg.inv(M_lock), np.array([0, 0, 1]))
        if i_c == 1:
            AxeZ = np.dot(M, A)
        else:
            AxeZ = np.dot(M2, A)

    thz = -s_z * np.float64(ui.angle_z_entry.text())
    if i_c == 1:
        M = np.dot(Rot(thz, AxeZ[0], AxeZ[1], AxeZ[2]), M)
    else:
        M2 = np.dot(Rot(thz, AxeZ[0], AxeZ[1], AxeZ[2]), M2)
    trace()
    euler_label()
    angle_z = angle_z - np.float64(ui.angle_z_entry.text())
    ui.angle_z_label_2.setText(str(angle_z))


def rot_z_p():
    global angle_beta, M, M2, angle_alpha, angle_z, var_lock, M_lock, s_z, t_ang
    i_c = crystal_check()

    if var_lock == 0:
        AxeZ = np.array([0, 0, 1])
    else:
        A = np.dot(np.linalg.inv(M_lock), np.array([0, 0, 1]))
        if i_c == 1:
            AxeZ = np.dot(M, A)
        else:
            AxeZ = np.dot(M2, A)

    thz = s_z * np.float64(ui.angle_z_entry.text())
    if i_c == 1:
        M = np.dot(Rot(thz, AxeZ[0], AxeZ[1], AxeZ[2]), M)
    else:
        M2 = np.dot(Rot(thz, AxeZ[0], AxeZ[1], AxeZ[2]), M2)
    trace()
    euler_label()

    angle_z = angle_z + np.float64(ui.angle_z_entry.text())
    ui.angle_z_label_2.setText(str(angle_z))


####################################################################
#
# Rotate around a given pole
#
####################################################################


def rotgm():
    global g, M, M2, Dstar, a, D
    i_c = crystal_check()
    thg = -np.float64(ui.rot_g_entry.text())
    diff = ui.pole_entry.text().split(",")
    diff1 = np.float64(diff[0])
    diff2 = np.float64(diff[1])
    diff3 = np.float64(diff[2])
    A = np.array([diff1, diff2, diff3])
    if var_uvw() == 1:
        if var_hexa(i_c) == 1:
            A = np.array([(2 * diff1 + diff2), (2 * diff2 + diff1), diff3])
        Ad = np.dot(D, A)
    else:
        Ad = np.dot(Dstar, A)
    if i_c == 1:
        Ap = np.dot(M, Ad) / np.linalg.norm(np.dot(M, Ad))
        M = np.dot(Rot(thg, Ap[0], Ap[1], Ap[2]), M)
    else:
        Ap = np.dot(M2, Ad) / np.linalg.norm(np.dot(M2, Ad))
        M2 = np.dot(Rot(thg, Ap[0], Ap[1], Ap[2]), M2)

    trace()
    euler_label()
    g = g - np.float64(ui.rot_g_entry.text())
    ui.rg_label.setText(str(g))
    return g, M, M2


def rotgp():
    global g, M, M2, Dstar, D
    i_c = crystal_check()
    thg = np.float64(ui.rot_g_entry.text())
    diff = ui.pole_entry.text().split(",")
    diff1 = np.float64(diff[0])
    diff2 = np.float64(diff[1])
    diff3 = np.float64(diff[2])
    A = np.array([diff1, diff2, diff3])
    if var_uvw() == 1:
        if var_hexa(i_c) == 1:
            A = np.array([(2 * diff1 + diff2), (2 * diff2 + diff1), diff3])
        Ad = np.dot(D, A)
    else:
        Ad = np.dot(Dstar, A)
    if i_c == 1:
        Ap = np.dot(M, Ad) / np.linalg.norm(np.dot(M, Ad))
        M = np.dot(Rot(thg, Ap[0], Ap[1], Ap[2]), M)
    else:
        Ap = np.dot(M2, Ad) / np.linalg.norm(np.dot(M2, Ad))
        M2 = np.dot(Rot(thg, Ap[0], Ap[1], Ap[2]), M2)
    trace()
    euler_label()
    g = g + np.float64(ui.rot_g_entry.text())
    ui.rg_label.setText(str(g))
    return g, M


####################################################################
#
# Add a given pole and equivalent ones
#
####################################################################

def pole(pole1, pole2, pole3):
    global M, M2, axes, axesh, T, V, D, Dstar
    i_c = crystal_check()
    if var_hexa(i_c) == 1:
        if var_uvw() == 1:
            pole1a = 2 * pole1 + pole2
            pole2a = 2 * pole2 + pole1
            pole1 = pole1a
            pole2 = pole2a

    Gs = np.array([pole1, pole2, pole3], float)

    if var_uvw() == 0:
        Gsh = np.dot(Dstar, Gs) / np.linalg.norm(np.dot(Dstar, Gs))
    else:
        Gsh = np.dot(D, Gs) / np.linalg.norm(np.dot(D, Gs))
    if i_c == 1:
        S = np.dot(M, Gsh)
    else:
        S = np.dot(M2, Gsh)
    if S[2] < 0:
        S = -S
        Gsh = -Gsh
        pole1 = -pole1
        pole2 = -pole2
        pole3 = -pole3

    T = np.vstack((T, np.array([S[0], S[1], S[2]])))
    axes = np.vstack((axes, np.array([pole1, pole2, pole3, i_c])))
    axes = np.vstack((axes, np.array([-pole1, -pole2, -pole3, i_c])))
    if var_uvw() == 0:
        axesh = np.vstack((axesh, np.array([Gsh[0], Gsh[1], Gsh[2], 0, color_trace(i_c), 0, 1, i_c, var_hexa(i_c), var_carre(i_c)])))
        axesh = np.vstack((axesh, np.array([-Gsh[0], -Gsh[1], -Gsh[2], 0, color_trace(i_c), 0, 1, i_c, var_hexa(i_c), var_carre(i_c)])))
    else:
        axesh = np.vstack((axesh, np.array([Gsh[0], Gsh[1], Gsh[2], 1, color_trace(i_c), 0, 1, i_c, var_hexa(i_c), var_carre(i_c)])))
        axesh = np.vstack((axesh, np.array([-Gsh[0], -Gsh[1], -Gsh[2], 1, color_trace(i_c), 0, 1, i_c, var_hexa(i_c), var_carre(i_c)])))

    return axes, axesh, T


def undo_pole(pole1, pole2, pole3):
    global M, M2, axes, axesh, T, V, D, Dstar

    i_c = crystal_check()
    if var_hexa(i_c) == 1:
        if var_uvw() == 1:
            pole1a = 2 * pole1 + pole2
            pole2a = 2 * pole2 + pole1
            pole1 = pole1a
            pole2 = pole2a

    Gs = np.array([pole1, pole2, pole3], float)

    if var_uvw() == 0:
        Gsh = np.dot(Dstar, Gs) / np.linalg.norm(np.dot(Dstar, Gs))
    else:
        Gsh = np.dot(D, Gs) / np.linalg.norm(np.dot(D, Gs))
    if i_c == 1:
        S = np.dot(M, Gsh)
    else:
        S = np.dot(M2, Gsh)
    if S[2] < 0:
        S = -S
        Gsh = -Gsh
        pole1 = -pole1
        pole2 = -pole2
        pole3 = -pole3

    ind = np.where((axes[:, 0] == pole1) & (axes[:, 1] == pole2) & (axes[:, 2] == pole3) & (axes[:, 3] == i_c) | (axes[:, 0] == -pole1) & (axes[:, 1] == -pole2) & (axes[:, 2] == -pole3) & (axes[:, 3] == i_c))
    axes = np.delete(axes, ind, 0)
    T = np.delete(T, ind, 0)
    axesh = np.delete(axesh, ind, 0)

    return axes, axesh, T


def d(pole1, pole2, pole3):
    global G

    ds = (np.sqrt(np.dot(np.array([pole1, pole2, pole3]), np.dot(np.linalg.inv(G), np.array([pole1, pole2, pole3])))))
    return ds


def addpole_sym():
    global M, M2, axes, axesh, T, V, D, Dstar, G
    i_c = crystal_check()
    pole_entry = ui.pole_entry.text().split(",")
    pole1 = np.float64(pole_entry[0])
    pole2 = np.float64(pole_entry[1])
    pole3 = np.float64(pole_entry[2])
    v = d(pole1, pole2, pole3)
    pole(pole1, pole2, pole3)

    if var_hexa(i_c) == 1:
        pole(pole1, pole2, pole3)
        pole(pole1, pole2, -pole3)
        pole(pole2, pole1, pole3)
        pole(pole2, pole1, -pole3)
        pole(-pole1 - pole2, pole2, pole3)
        pole(-pole1 - pole2, pole2, -pole3)
        pole(pole1, -pole1 - pole2, pole3)
        pole(pole1, -pole1 - pole2, -pole3)
        pole(pole2, -pole1 - pole2, pole3)
        pole(pole2, -pole1 - pole2, -pole3)
        pole(-pole1 - pole2, pole1, pole3)
        pole(-pole1 - pole2, pole1, -pole3)

    else:
        if np.abs(d(pole1, pole2, -pole3) - v) < 0.001:
            pole(pole1, pole2, -pole3)
        if np.abs(d(pole1, -pole2, pole3) - v) < 0.001:
            pole(pole1, -pole2, pole3)
        if np.abs(d(-pole1, pole2, pole3) - v) < 0.001:
            pole(-pole1, pole2, pole3)
        if np.abs(d(pole2, pole1, pole3) - v) < 0.001:
            pole(pole2, pole1, pole3)
        if np.abs(d(pole2, pole1, -pole3) - v) < 0.001:
            pole(pole2, pole1, -pole3)
        if np.abs(d(pole2, -pole1, pole3) - v) < 0.001:
            pole(pole2, -pole1, pole3)
        if np.abs(d(-pole2, pole1, pole3) - v) < 0.001:
            pole(-pole2, pole1, pole3)
        if np.abs(d(pole2, pole3, pole1) - v) < 0.001:
            pole(pole2, pole3, pole1)
        if np.abs(d(pole2, pole3, -pole1) - v) < 0.001:
            pole(pole2, pole3, -pole1)
        if np.abs(d(pole2, -pole3, pole1) - v) < 0.001:
            pole(pole2, -pole3, pole1)
        if np.abs(d(-pole2, pole3, pole1) - v) < 0.001:
            pole(-pole2, pole3, pole1)
        if np.abs(d(pole1, pole3, pole2) - v) < 0.001:
            pole(pole1, pole3, pole2)
        if np.abs(d(pole1, pole3, -pole2) - v) < 0.001:
            pole(pole1, pole3, -pole2)
        if np.abs(d(pole1, -pole3, pole2) - v) < 0.001:
            pole(pole1, -pole3, pole2)
        if np.abs(d(-pole1, pole3, pole2) - v) < 0.001:
            pole(-pole1, pole3, pole2)
        if np.abs(d(pole3, pole1, pole2) - v) < 0.001:
            pole(pole3, pole1, pole2)
        if np.abs(d(pole3, pole1, -pole2) - v) < 0.001:
            pole(pole3, pole1, -pole2)
        if np.abs(d(pole3, -pole1, pole2) - v) < 0.001:
            pole(pole3, -pole1, pole2)
        if np.abs(d(-pole3, pole1, pole2) - v) < 0.001:
            pole(-pole3, pole1, pole2)
        if np.abs(d(pole3, pole2, pole1) - v) < 0.001:
            pole(pole3, pole2, pole1)
        if np.abs(d(pole3, pole2, -pole1) - v) < 0.001:
            pole(pole3, pole2, -pole1)
        if np.abs(d(pole3, -pole2, pole1) - v) < 0.001:
            pole(pole3, -pole2, pole1)
        if np.abs(d(-pole3, pole2, pole1) - v) < 0.001:
            pole(-pole3, pole2, pole1)
    trace()


def undo_sym():
    global M, M2, axes, axesh, T, V, D, Dstar, G
    i_c = crystal_check()
    pole_entry = ui.pole_entry.text().split(",")
    pole1 = np.float64(pole_entry[0])
    pole2 = np.float64(pole_entry[1])
    pole3 = np.float64(pole_entry[2])
    v = d(pole1, pole2, pole3)
    undo_pole(pole1, pole2, pole3)
    if var_hexa(i_c) == 1:
        undo_pole(pole1, pole2, pole3)
        undo_pole(pole1, pole2, -pole3)
        undo_pole(pole2, pole1, pole3)
        undo_pole(pole2, pole1, -pole3)
        undo_pole(-pole1 - pole2, pole2, pole3)
        undo_pole(-pole1 - pole2, pole2, -pole3)
        undo_pole(pole1, -pole1 - pole2, pole3)
        undo_pole(pole1, -pole1 - pole2, -pole3)
        undo_pole(pole2, -pole1 - pole2, pole3)
        undo_pole(pole2, -pole1 - pole2, -pole3)
        undo_pole(-pole1 - pole2, pole1, pole3)
        undo_pole(-pole1 - pole2, pole1, -pole3)

    else:
        if np.abs(d(pole1, pole2, -pole3) - v) < 0.001:
            undo_pole(pole1, pole2, -pole3)
        if np.abs(d(pole1, -pole2, pole3) - v) < 0.001:
            undo_pole(pole1, -pole2, pole3)
        if np.abs(d(-pole1, pole2, pole3) - v) < 0.001:
            undo_pole(-pole1, pole2, pole3)
        if np.abs(d(pole2, pole1, pole3) - v) < 0.001:
            undo_pole(pole2, pole1, pole3)
        if np.abs(d(pole2, pole1, -pole3) - v) < 0.001:
            undo_pole(pole2, pole1, -pole3)
        if np.abs(d(pole2, -pole1, pole3) - v) < 0.001:
            undo_pole(pole2, -pole1, pole3)
        if np.abs(d(-pole2, pole1, pole3) - v) < 0.001:
            undo_pole(-pole2, pole1, pole3)
        if np.abs(d(pole2, pole3, pole1) - v) < 0.001:
            undo_pole(pole2, pole3, pole1)
        if np.abs(d(pole2, pole3, -pole1) - v) < 0.001:
            undo_pole(pole2, pole3, -pole1)
        if np.abs(d(pole2, -pole3, pole1) - v) < 0.001:
            undo_pole(pole2, -pole3, pole1)
        if np.abs(d(-pole2, pole3, pole1) - v) < 0.001:
            undo_pole(-pole2, pole3, pole1)
        if np.abs(d(pole1, pole3, pole2) - v) < 0.001:
            undo_pole(pole1, pole3, pole2)
        if np.abs(d(pole1, pole3, -pole2) - v) < 0.001:
            undo_pole(pole1, pole3, -pole2)
        if np.abs(d(pole1, -pole3, pole2) - v) < 0.001:
            undo_pole(pole1, -pole3, pole2)
        if np.abs(d(-pole1, pole3, pole2) - v) < 0.001:
            undo_pole(-pole1, pole3, pole2)
        if np.abs(d(pole3, pole1, pole2) - v) < 0.001:
            undo_pole(pole3, pole1, pole2)
        if np.abs(d(pole3, pole1, -pole2) - v) < 0.001:
            undo_pole(pole3, pole1, -pole2)
        if np.abs(d(pole3, -pole1, pole2) - v) < 0.001:
            undo_pole(pole3, -pole1, pole2)
        if np.abs(d(-pole3, pole1, pole2) - v) < 0.001:
            undo_pole(-pole3, pole1, pole2)
        if np.abs(d(pole3, pole2, pole1) - v) < 0.001:
            undo_pole(pole3, pole2, pole1)
        if np.abs(d(pole3, pole2, -pole1) - v) < 0.001:
            undo_pole(pole3, pole2, -pole1)
        if np.abs(d(pole3, -pole2, pole1) - v) < 0.001:
            undo_pole(pole3, -pole2, pole1)
        if np.abs(d(-pole3, pole2, pole1) - v) < 0.001:
            undo_pole(-pole3, pole2, pole1)
    trace()


def addpole():

    pole_entry = ui.pole_entry.text().split(",")
    pole1 = np.float64(pole_entry[0])
    pole2 = np.float64(pole_entry[1])
    pole3 = np.float64(pole_entry[2])
    pole(pole1, pole2, pole3)
    trace()


def undo_addpole():
    global M, M2, axes, axesh, T, V, D, Dstar, trP
    pole_entry = ui.pole_entry.text().split(",")
    pole1 = np.float64(pole_entry[0])
    pole2 = np.float64(pole_entry[1])
    pole3 = np.float64(pole_entry[2])
    undo_pole(pole1, pole2, pole3)
    trace()

####################################################################
#
# Plot a given plane and equivalent ones. Plot a cone
#
####################################################################


def trace_plan(pole1, pole2, pole3):
    global M, axes, axesh, T, V, D, Dstar, trP, trC
    i_c = crystal_check()
    pole_i = 0
    pole_c = color_trace(i_c)

    if var_hexa(i_c) == 1:
        if var_uvw() == 1:
            pole1 = 2 * pole1 + pole2
            pole2 = 2 * pole2 + pole1
            pole_i = 1
    trP = np.vstack((trP, np.array([pole1, pole2, pole3, pole_i, pole_c, i_c])))
    b = np.ascontiguousarray(trP).view(np.dtype((np.void, trP.dtype.itemsize * trP.shape[1])))
    trP = np.unique(b).view(trP.dtype).reshape(-1, trP.shape[1])


def trace_addplan():
    global M, M2, axes, axesh, T, V, D, Dstar, trP

    pole_entry = ui.pole_entry.text().split(",")
    pole1 = np.float64(pole_entry[0])
    pole2 = np.float64(pole_entry[1])
    pole3 = np.float64(pole_entry[2])

    trace_plan(pole1, pole2, pole3)
    trace()


def undo_trace_addplan():
    global M, M2, axes, axesh, T, V, D, Dstar, trP

    pole_entry = ui.pole_entry.text().split(",")
    pole1 = np.float64(pole_entry[0])
    pole2 = np.float64(pole_entry[1])
    pole3 = np.float64(pole_entry[2])

    undo_trace_plan(pole1, pole2, pole3)
    trace()


def undo_trace_plan(pole1, pole2, pole3):
    global M, M2, axes, axesh, T, V, D, Dstar, trP, tr_schmid
    i_c = crystal_check()
    ind = np.where((trP[:, 0] == pole1) & (trP[:, 1] == pole2) & (trP[:, 2] == pole3) | (trP[:, 0] == -pole1) & (trP[:, 1] == -pole2) & (trP[:, 2] == -pole3) & (trP[:, 3] == i_c))

    trP = np.delete(trP, ind, 0)
    b = np.ascontiguousarray(trP).view(np.dtype((np.void, trP.dtype.itemsize * trP.shape[1])))
    trP = np.unique(b).view(trP.dtype).reshape(-1, trP.shape[1])


def trace_plan_sym():
    global M, M2, axes, axesh, T, V, D, Dstar, G
    i_c = crystal_check()
    pole_entry = ui.pole_entry.text().split(",")
    pole1 = np.float64(pole_entry[0])
    pole2 = np.float64(pole_entry[1])
    pole3 = np.float64(pole_entry[2])
    v = d(pole1, pole2, pole3)
    trace_plan(pole1, pole2, pole3)

    if var_hexa(i_c) == 1:
        trace_plan(pole1, pole2, pole3)
        trace_plan(pole1, pole2, -pole3)
        trace_plan(pole2, pole1, pole3)
        trace_plan(pole2, pole1, -pole3)
        trace_plan(-pole1 - pole2, pole2, pole3)
        trace_plan(-pole1 - pole2, pole2, -pole3)
        trace_plan(pole1, -pole1 - pole2, pole3)
        trace_plan(pole1, -pole1 - pole2, -pole3)
        trace_plan(pole2, -pole1 - pole2, pole3)
        trace_plan(pole2, -pole1 - pole2, -pole3)
        trace_plan(-pole1 - pole2, pole1, pole3)
        trace_plan(-pole1 - pole2, pole1, -pole3)

    else:
        if np.abs(d(pole1, pole2, -pole3) - v) < 0.001:
            trace_plan(pole1, pole2, -pole3)
        if np.abs(d(pole1, -pole2, pole3) - v) < 0.001:
            trace_plan(pole1, -pole2, pole3)
        if np.abs(d(-pole1, pole2, pole3) - v) < 0.001:
            trace_plan(-pole1, pole2, pole3)
        if np.abs(d(pole2, pole1, pole3) - v) < 0.001:
            trace_plan(pole2, pole1, pole3)
        if np.abs(d(pole2, pole1, -pole3) - v) < 0.001:
            trace_plan(pole2, pole1, -pole3)
        if np.abs(d(pole2, -pole1, pole3) - v) < 0.001:
            trace_plan(pole2, -pole1, pole3)
        if np.abs(d(-pole2, pole1, pole3) - v) < 0.001:
            trace_plan(-pole2, pole1, pole3)
        if np.abs(d(pole2, pole3, pole1) - v) < 0.001:
            trace_plan(pole2, pole3, pole1)
        if np.abs(d(pole2, pole3, -pole1) - v) < 0.001:
            trace_plan(pole2, pole3, -pole1)
        if np.abs(d(pole2, -pole3, pole1) - v) < 0.001:
            trace_plan(pole2, -pole3, pole1)
        if np.abs(d(-pole2, pole3, pole1) - v) < 0.001:
            trace_plan(-pole2, pole3, pole1)
        if np.abs(d(pole1, pole3, pole2) - v) < 0.001:
            trace_plan(pole1, pole3, pole2)
        if np.abs(d(pole1, pole3, -pole2) - v) < 0.001:
            trace_plan(pole1, pole3, -pole2)
        if np.abs(d(pole1, -pole3, pole2) - v) < 0.001:
            trace_plan(pole1, -pole3, pole2)
        if np.abs(d(-pole1, pole3, pole2) - v) < 0.001:
            trace_plan(-pole1, pole3, pole2)
        if np.abs(d(pole3, pole1, pole2) - v) < 0.001:
            trace_plan(pole3, pole1, pole2)
        if np.abs(d(pole3, pole1, -pole2) - v) < 0.001:
            trace_plan(pole3, pole1, -pole2)
        if np.abs(d(pole3, -pole1, pole2) - v) < 0.001:
            trace_plan(pole3, -pole1, pole2)
        if np.abs(d(-pole3, pole1, pole2) - v) < 0.001:
            trace_plan(-pole3, pole1, pole2)
        if np.abs(d(pole3, pole2, pole1) - v) < 0.001:
            trace_plan(pole3, pole2, pole1)
        if np.abs(d(pole3, pole2, -pole1) - v) < 0.001:
            trace_plan(pole3, pole2, -pole1)
        if np.abs(d(pole3, -pole2, pole1) - v) < 0.001:
            trace_plan(pole3, -pole2, pole1)
        if np.abs(d(-pole3, pole2, pole1) - v) < 0.001:
            trace_plan(-pole3, pole2, pole1)
    trace()


def undo_trace_plan_sym():
    global M, M2, axes, axesh, T, V, D, Dstar, G
    i_c = crystal_check()
    pole_entry = ui.pole_entry.text().split(",")
    pole1 = np.float64(pole_entry[0])
    pole2 = np.float64(pole_entry[1])
    pole3 = np.float64(pole_entry[2])
    v = d(pole1, pole2, pole3)

    undo_trace_plan(pole1, pole2, pole3)
    if var_hexa(i_c) == 1:
        undo_trace_plan(pole1, pole2, pole3)
        undo_trace_plan(pole1, pole2, -pole3)
        undo_trace_plan(pole2, pole1, pole3)
        undo_trace_plan(pole2, pole1, -pole3)
        undo_trace_plan(-pole1 - pole2, pole2, pole3)
        undo_trace_plan(-pole1 - pole2, pole2, -pole3)
        undo_trace_plan(pole1, -pole1 - pole2, pole3)
        undo_trace_plan(pole1, -pole1 - pole2, -pole3)
        undo_trace_plan(pole2, -pole1 - pole2, pole3)
        undo_trace_plan(pole2, -pole1 - pole2, -pole3)
        undo_trace_plan(-pole1 - pole2, pole1, pole3)
        undo_trace_plan(-pole1 - pole2, pole1, -pole3)

    else:
        if np.abs(d(pole1, pole2, -pole3) - v) < 0.001:
            undo_trace_plan(pole1, pole2, -pole3)
        if np.abs(d(pole1, -pole2, pole3) - v) < 0.001:
            undo_trace_plan(pole1, -pole2, pole3)
        if np.abs(d(-pole1, pole2, pole3) - v) < 0.001:
            undo_trace_plan(-pole1, pole2, pole3)
        if np.abs(d(pole2, pole1, pole3) - v) < 0.001:
            undo_trace_plan(pole2, pole1, pole3)
        if np.abs(d(pole2, pole1, -pole3) - v) < 0.001:
            undo_trace_plan(pole2, pole1, -pole3)
        if np.abs(d(pole2, -pole1, pole3) - v) < 0.001:
            undo_trace_plan(pole2, -pole1, pole3)
        if np.abs(d(-pole2, pole1, pole3) - v) < 0.001:
            undo_trace_plan(-pole2, pole1, pole3)
        if np.abs(d(pole2, pole3, pole1) - v) < 0.001:
            undo_trace_plan(pole2, pole3, pole1)
        if np.abs(d(pole2, pole3, -pole1) - v) < 0.001:
            undo_trace_plan(pole2, pole3, -pole1)
        if np.abs(d(pole2, -pole3, pole1) - v) < 0.001:
            undo_trace_plan(pole2, -pole3, pole1)
        if np.abs(d(-pole2, pole3, pole1) - v) < 0.001:
            undo_trace_plan(-pole2, pole3, pole1)
        if np.abs(d(pole1, pole3, pole2) - v) < 0.001:
            undo_trace_plan(pole1, pole3, pole2)
        if np.abs(d(pole1, pole3, -pole2) - v) < 0.001:
            undo_trace_plan(pole1, pole3, -pole2)
        if np.abs(d(pole1, -pole3, pole2) - v) < 0.001:
            undo_trace_plan(pole1, -pole3, pole2)
        if np.abs(d(-pole1, pole3, pole2) - v) < 0.001:
            undo_trace_plan(-pole1, pole3, pole2)
        if np.abs(d(pole3, pole1, pole2) - v) < 0.001:
            undo_trace_plan(pole3, pole1, pole2)
        if np.abs(d(pole3, pole1, -pole2) - v) < 0.001:
            undo_trace_plan(pole3, pole1, -pole2)
        if np.abs(d(pole3, -pole1, pole2) - v) < 0.001:
            undo_trace_plan(pole3, -pole1, pole2)
        if np.abs(d(-pole3, pole1, pole2) - v) < 0.001:
            undo_trace_plan(-pole3, pole1, pole2)
        if np.abs(d(pole3, pole2, pole1) - v) < 0.001:
            undo_trace_plan(pole3, pole2, pole1)
        if np.abs(d(pole3, pole2, -pole1) - v) < 0.001:
            undo_trace_plan(pole3, pole2, -pole1)
        if np.abs(d(pole3, -pole2, pole1) - v) < 0.001:
            undo_trace_plan(pole3, -pole2, pole1)
        if np.abs(d(-pole3, pole2, pole1) - v) < 0.001:
            undo_trace_plan(-pole3, pole2, pole1)
    trace()


def trace_plan2(B):
    global M, M2, axes, axesh, T, V, D, Dstar, a, trP

    for h in range(0, B.shape[0]):
        pole1 = B[h, 0]
        pole2 = B[h, 1]
        pole3 = B[h, 2]
        Gs = np.array([pole1, pole2, pole3], float)

        if B[h, 3] == 0:
            Gsh = np.dot(Dstar, Gs) / np.linalg.norm(np.dot(Dstar, Gs))
        else:
            Gsh = np.dot(D, Gs) / np.linalg.norm(np.dot(D, Gs))
        if B[h, 4] == 1:
            S = np.dot(M, Gsh)
        else:
            S = np.dot(M2, Gsh)
        if S[2] < 0:
            S = -S
            Gsh = -Gsh
            pole1 = -pole1
            pole2 = -pole2
            pole3 = -pole3
        r = np.sqrt(S[0]**2 + S[1]**2 + S[2]**2)
        A = np.zeros((2, 100))
        Q = np.zeros((1, 2))
        t = np.arctan2(S[1], S[0]) * 180 / np.pi
        w = 0
        ph = np.arccos(S[2] / r) * 180 / np.pi

        for g in np.linspace(-np.pi, np.pi, 100):
            Aa = np.dot(Rot(t, 0, 0, 1), np.dot(Rot(ph, 0, 1, 0), np.array([np.sin(g), np.cos(g), 0])))
            if np.sign(Aa[2]) < 0 and np.abs(Aa[2]) < 1e-8:
                Aa = -Aa
            A[:, w] = proj(Aa[0], Aa[1], Aa[2]) * 300
            Q = np.vstack((Q, A[:, w]))
            w = w + 1

        Q = np.delete(Q, 0, 0)
        Q = Q[~np.isnan(Q).any(axis=1)]
        if B[h, 4] == 1:
            a.plot(Q[:, 0] + 300, Q[:, 1] + 300, 'b')
        if B[h, 4] == 2:
            a.plot(Q[:, 0] + 300, Q[:, 1] + 300, 'g')
        if B[h, 4] == 3:
            a.plot(Q[:, 0] + 300, Q[:, 1] + 300, 'r')


####################################################################
#
# Click a pole
#
####################################################################

def click_a_pole(event):

    global M, M2, Dstar, D, minx, maxx, miny, maxy, a, Stc
    i_c = crystal_check()
    if event.button == 3:
        x = event.xdata
        y = event.ydata

        x = (x - 300) / 300
        y = (y - 300) / 300
        X = 2 * x / (1 + x**2 + y**2)
        Y = 2 * y / (1 + x**2 + y**2)
        Z = (-1 + x**2 + y**2) / (1 + x**2 + y**2)
        if Z < 0:
            X = -X
            Y = -Y
        if i_c == 1:
            A = np.dot(np.linalg.inv(M), np.array([X, Y, Z]))
        else:
            A = np.dot(np.linalg.inv(M2), np.array([X, Y, Z]))
        if var_uvw() == 0:
            A = np.dot(np.linalg.inv(Dstar), A) * 1e10 * 100
        else:
            A = np.dot(np.linalg.inv(D), A) * 1e-10 * 100
            if var_hexa(i_c) == 1:
                Aa = (2 * A[0] - A[1]) / 3
                Ab = (2 * A[1] - A[0]) / 3
                A[0] = Aa
                A[1] = Ab

        pole(A[0], A[1], A[2])
        Stc = np.vstack((Stc, np.array([A[0], A[1], A[2]])))
        trace()


def undo_click_a_pole():
    global Stc
    undo_pole(Stc[-1, 0], Stc[-1, 1], Stc[-1, 2])
    Stc = Stc[:-1, :]
    trace()

####################################################################
#
# Inclinaison-beta indicator when the mouse is on the stereo
#
####################################################################


def coordinates(event):
    t_ang = ang_work_space() * np.pi / 180
    if event.xdata and event.ydata:
        x = event.xdata
        y = event.ydata
        x = (x - 300) / 300
        y = (y - 300) / 300
        X0 = 2 * x / (1 + x**2 + y**2)
        Y0 = 2 * y / (1 + x**2 + y**2)
        Z0 = (-1 + x**2 + y**2) / (1 + x**2 + y**2)
        Rxyz = np.dot(Rot(t_ang * 180 / np.pi, 0, 0, 1), [X0, Y0, Z0])
        X = Rxyz[0]
        Y = Rxyz[1]
        Z = Rxyz[2]
        lat = np.arctan2(np.sqrt(X**2 + Z**2), Y) * 180 / np.pi
        if X < 0:
            lat = -lat
        longi = -np.arctan2(Z, X) * 180 / np.pi
        if ui.alpha_signBox.isChecked():
            longi = -longi
        if np.abs(longi) > 90:
            if longi > 0:
                longi = longi - 180
            else:
                longi = longi + 180

        c = str(np.around(longi, decimals=1)) + str(',') + str(np.around(lat, decimals=1))
        ui.coord_label.setText(str(c))


####################################################################
#
# Reset view after zoom/update axes/angles
#
####################################################################


def reset_view():
    global a

    a.axis([minx, maxx, miny, maxy])
    mpl.rcParams['font.size'] = ui.text_size_entry.text()
    trace()


def tilt_axes():
    global s_a, s_b, s_z
    s_a, s_b, s_z = 1, 1, 1
    if ui.alpha_signBox.isChecked():
        s_a = -1
    if ui.beta_signBox.isChecked():
        s_b = -1
    if ui.theta_signBox.isChecked():
        s_b = -1
    return s_a, s_b, s_z

####################################################################
#
# Enable or disable Wulff net
#
####################################################################


def wulff():
    global a
    if ui.wulff_button.isChecked():
        fn = os.path.join(os.path.dirname(__file__), 'stereo.png')
        img = Image.open(fn)
        img = img.rotate(float(ang_work_space()), fillcolor='white')
        img = np.array(img)
    else:
        img = 255 * np.ones([600, 600, 3], dtype=np.uint8)
        circle = plt.Circle((300, 300), 300, color='black', fill=False)
        a.add_artist(circle)
        a.plot(300, 300, '+', markersize=10, mew=3, color='black')

    a.imshow(img, interpolation="bicubic")
    a.axis('off')
    plt.tight_layout()
    plt.subplots_adjust(top=0.99, bottom=0.01, right=0.9, left=0.1)
    a.figure.canvas.draw()


def text_label(A, B):
    Aa = A[0]
    Ab = A[1]
    Ac = A[2]

    if B[3] == 1 & int(B[8]) == 1:
        Aa = (2 * A[0] - A[1])
        Ab = (2 * A[1] - A[0])
        Ac = 3 * A[2]
        m = functools.reduce(lambda x, y: GCD(x, y), [Aa, Ab, Ac])
        if np.abs(m) > 1e-3:
            Aa = Aa / m
            Ab = Ab / m
            Ac = Ac / m

    if np.sign(Aa) < 0:
        s0 = r'$\overline{' + str(np.abs(int(Aa))) + '}$'
    else:
        s0 = str(np.abs(int(Aa)))
    if np.sign(Ab) < 0:
        s1 = r'$\overline{' + str(np.abs(int(Ab))) + '}$'
    else:
        s1 = str(np.abs(int(Ab)))
    if np.sign(Ac) < 0:
        s2 = r'$\overline{' + str(np.abs(int(Ac))) + '}$'
    else:
        s2 = str(np.abs(int(Ac)))
    s = s0 + ',' + s1 + ',' + s2
    if int(B[8]) == 1:
        if np.sign(-Aa - Ab) < 0:
            s3 = r'$\overline{' + str(int(np.abs(-Aa - Ab))) + '}$'
        else:
            s3 = str(int(np.abs(-Aa - Ab)))
        s = s0 + ',' + s1 + ',' + s3 + ',' + s2
    if B[3] == 1:
        s = '[' + s + ']'
    return s


def crystal_check():
    global D, Dstar, V, G
    if ui.crystal2_radioButton.isChecked():
        i_c = 2
        abc = ui.abc_entry_2.text().split(",")
        alphabetagamma = ui.alphabetagamma_entry_2.text().split(",")
        D, Dstar, V, G = crist_mat(abc, alphabetagamma)
    else:
        i_c = 1
        abc = ui.abc_entry.text().split(",")
        alphabetagamma = ui.alphabetagamma_entry.text().split(",")
        D, Dstar, V, G = crist_mat(abc, alphabetagamma)
    return i_c


########################################################
# Main
#
#######################################################

########################################################
# Refresh action on stereo
#
#######################################################

def trace():
    global T, x, y, z, axes, axesh, M, M2, trP, a, trC, s_a, s_b, s_z, Qp, D1, D0, S
    minx, maxx = a.get_xlim()
    miny, maxy = a.get_ylim()
    a.clear()
    P = np.zeros((axes.shape[0], 2))
    T = np.zeros((axes.shape[0], 3))
    trace_plan2(trP)
    C = []
    O = []
    tilt_axes()
    for i in range(0, axes.shape[0]):
        if axesh[i, 6] == 1:
            axeshr = np.array([axesh[i, 0], axesh[i, 1], axesh[i, 2]])
            if axesh[i, 7] == 1:
                T[i, :] = np.dot(M, axeshr)
            else:
                T[i, :] = np.dot(M2, axeshr)
            P[i, :] = proj(T[i, 0], T[i, 1], T[i, 2]) * 300
            s = text_label(axes[i, :], axesh[i, :])
            a.annotate(s, (P[i, 0] + 300, P[i, 1] + 300))
        if axesh[i, 4] == 1:
            C.append('b')
        if axesh[i, 4] == 2:
            C.append('g')
        if axesh[i, 4] == 3:
            C.append('r')
        if axesh[i, 9] == 1:
            O.append('none')
        if axesh[i, 9] == 0:
            O.append(C[-1])

    s0 = axesh[:, 6]
    a.scatter(P[:, 0] + 300, P[:, 1] + 300, edgecolors=C, s=s0 * np.float64(ui.size_var.text()), facecolors=O, linewidth=1.5)
    trace_misorientation(Qp)
    a.axis([minx, maxx, miny, maxy])
    wulff()


# "
#
# Plot from Euler angle
#
# "

def princ2():
    global T, angle_alpha, angle_beta, angle_z, M, M2, Dstar, D, g, M0, trP, a, axeshr, minx, maxx, miny, maxy, trC, Stc, dmip, tr_schmid, s_a, s_b, s_c, axes, axesh, D, Dstar, V, G, Qp, a

    a.clear()
    trP = np.zeros((1, 6))
    Stc = np.zeros((1, 3))
    Qp = np.zeros((1, 2))

    phi1phiphi2 = ui.phi1phiphi2_entry.text().split(",")
    phi1 = np.float64(phi1phiphi2[0])
    phi = np.float64(phi1phiphi2[1])
    phi2 = np.float64(phi1phiphi2[2])

    phi1phiphi2_2 = ui.phi1phiphi2_2_entry.text().split(",")
    phi1_2 = np.float64(phi1phiphi2_2[0])
    phi_2 = np.float64(phi1phiphi2_2[1])
    phi2_2 = np.float64(phi1phiphi2_2[2])

    crist()
    tilt_axes()

    P = np.zeros((axes.shape[0], 2))
    T = np.zeros((axes.shape[0], 3))
    C = []
    O = []

    for i in range(0, axes.shape[0]):
        axeshr = np.array([axesh[i, 0], axesh[i, 1], axesh[i, 2]])
        if axesh[i, 5] != -1 and axesh[i, 6] == 1:
            if axesh[i, 7] == 1:
                T[i, :] = np.dot(rotation(phi1, phi, phi2), axeshr)
            else:
                T[i, :] = np.dot(rotation(phi1_2, phi_2, phi2_2), axeshr)

            P[i, :] = proj(T[i, 0], T[i, 1], T[i, 2]) * 300
            s = text_label(axes[i, :], axesh[i, :])
            a.annotate(s, (P[i, 0] + 300, P[i, 1] + 300))
        if axesh[i, 4] == 1:
            C.append('b')
        if axesh[i, 4] == 2:
            C.append('g')
        if axesh[i, 4] == 3:
            C.append('r')
        if axesh[i, 9] == 1:
            O.append('none')
        if axesh[i, 9] == 0:
            O.append(C[-1])

    s0 = axesh[:, 6]
    a.scatter(P[:, 0] + 300, P[:, 1] + 300, edgecolors=C, s=s0 * np.float64(ui.size_var.text()), facecolors=O, linewidth=1.5)

    minx, maxx = -2, 602
    miny, maxy = -2, 602
    a.axis([minx, maxx, miny, maxy])
    wulff()

    angle_alpha = 0
    angle_beta = 0
    angle_z = 0
    g = 0
    ui.angle_alpha_label_2.setText('0.0')
    ui.angle_beta_label_2.setText('0.0')
    ui.angle_z_label_2.setText('0.0')
    ui.angle_beta_label_2.setText('0.0')
    ui.angle_z_label_2.setText('0.0')
    M = rotation(phi1, phi, phi2)
    M2 = rotation(phi1_2, phi_2, phi2_2)
    t = str(np.around(phi1, decimals=1)) + str(',') + str(np.around(phi, decimals=1)) + str(',') + str(np.around(phi2, decimals=1))
    t2 = str(np.around(phi1_2, decimals=1)) + str(',') + str(np.around(phi_2, decimals=1)) + str(',') + str(np.around(phi2_2, decimals=1))
    ui.angle_euler_label.setText(t)
    ui.angle_euler_label_2.setText(t2)

    return T, angle_alpha, angle_beta, angle_z, g, M, M2

#####################################
#
# Misorientation
#
#####################################


def check_gb():
    abc2 = ui.abc_entry_2.text().split(",")
    alphabetagamma2 = ui.alphabetagamma_entry_2.text().split(",")
    abc = ui.abc_entry.text().split(",")
    alphabetagamma = ui.alphabetagamma_entry.text().split(",")
    if abc == abc2 and alphabetagamma == alphabetagamma2:
        gb = 1
    else:
        gb = 2
    return gb


def Rota(t, u, v, w, g):
    Ae = np.dot(g, np.array([u, v, w]))
    Re = Rot(t, Ae[0], Ae[1], Ae[2])
    return Re


def cryststruct():
    global cs
    gb = check_gb()
    if gb == 1:
        abc = ui.abc_entry.text().split(",")
        alphabetagamma = ui.alphabetagamma_entry.text().split(",")
    else:
        return
    a = np.float64(abc[0]) * 1e-10
    b = np.float64(abc[1]) * 1e-10
    c = np.float64(abc[2]) * 1e-10
    alp = np.float64(alphabetagamma[0])
    bet = np.float64(alphabetagamma[1])
    gam = np.float64(alphabetagamma[2])

    if gam == 90 and alp == 90 and bet == 90 and a == b and b == c:
        cs = 1

    if gam == 120 and alp == 90 and bet == 90:
        cs = 2

    if gam == 90 and alp == 90 and bet == 90 and a == b and b != c:
        cs = 3

    if alp != 90 and a == b and b == c:
        cs = 4

    if gam == 90 and alp == 90 and bet == 90 and a != b and b != c:
        cs = 5

    if gam != 90 and alp == 90 and bet == 90 and a != b and b != c:
        cs = 6

    if gam != alp != bet and a != b != c:
        cs = 7
    return cs


def Sy(g):
    global cs
    if cs == 1:
        S1 = Rota(90, 1, 0, 0, g)
        S2 = Rota(180, 1, 0, 0, g)
        S3 = Rota(270, 1, 0, 0, g)
        S4 = Rota(90, 0, 1, 0, g)
        S5 = Rota(180, 0, 1, 0, g)
        S6 = Rota(270, 0, 1, 0, g)
        S7 = Rota(90, 0, 0, 1, g)
        S8 = Rota(180, 0, 0, 1, g)
        S9 = Rota(270, 0, 0, 1, g)
        S10 = Rota(180, 1, 1, 0, g)
        S11 = Rota(180, 1, 0, 1, g)
        S12 = Rota(180, 0, 1, 1, g)
        S13 = Rota(180, -1, 1, 0, g)
        S14 = Rota(180, -1, 0, 1, g)
        S15 = Rota(180, 0, -1, 1, g)
        S16 = Rota(120, 1, 1, 1, g)
        S17 = Rota(240, 1, 1, 1, g)
        S18 = Rota(120, -1, 1, 1, g)
        S19 = Rota(240, -1, 1, 1, g)
        S20 = Rota(120, 1, -1, 1, g)
        S21 = Rota(240, 1, -1, 1, g)
        S22 = Rota(120, 1, 1, -1, g)
        S23 = Rota(240, 1, 1, -1, g)
        S24 = np.eye(3, 3)
        S = np.vstack((S1, S2, S3, S4, S5, S6, S7, S8, S9, S10, S11, S12, S13, S14, S15, S16, S17, S18, S19, S20, S21, S22, S23, S24))

    if cs == 2:
        S1 = Rota(60, 0, 0, 1, g)
        S2 = Rota(120, 0, 0, 1, g)
        S3 = Rota(180, 0, 0, 1, g)
        S4 = Rota(240, 0, 0, 1, g)
        S5 = Rota(300, 0, 0, 1, g)
        S6 = np.eye(3, 3)
        S7 = Rota(180, 1, 0, 0, g)
        S8 = Rota(180, 0, 1, 0, g)
        S9 = Rota(180, 1 / 2, np.sqrt(3) / 2, 0, g)
        S10 = Rota(180, -1 / 2, np.sqrt(3) / 2, 0, g)
        S11 = Rota(180, np.sqrt(3) / 2, 1 / 2, 0, g)
        S12 = Rota(180, -np.sqrt(3) / 2, 1 / 2, 0, g)
        S = np.vstack((S1, S2, S3, S4, S5, S6, S7, S8, S9, S10, S11, S12))

    if cs == 3:
        S1 = Rota(90, 0, 0, 1, g)
        S2 = Rota(180, 0, 0, 1, g)
        S3 = Rota(270, 0, 0, 1, g)
        S4 = Rota(180, 0, 1, 0, g)
        S5 = Rota(180, 1, 0, 0, g)
        S6 = Rota(180, 1, 1, 0, g)
        S7 = Rota(180, 1, -1, 0, g)
        S8 = np.eye(3, 3)
        S = np.vstack((S1, S2, S3, S4, S5, S6, S7, S8))

    if cs == 4:
        n1 = np.dot(Dstar, [1, 1, 1])
        n2 = np.dot(Dstar, [0, -1, 1])
        n3 = np.dot(Dstar, [-1, 0, 1])
        n4 = np.dot(Dstar, [1, -1, 0])
        S1 = Rota(120, n1[0], n1[1], n1[2], g)
        S2 = Rota(240, n1[0], n1[1], n1[2], g)
        S3 = Rota(180, n2[0], n2[1], n2[2], g)
        S4 = Rota(180, n3[0], n3[1], n3[2], g)
        S5 = Rota(180, n4[0], n4[1], n4[2], g)
        S6 = np.eye(3, 3)
        S = np.vstack((S1, S2, S3, S4, S5, S6))

    if cs == 5:
        S1 = Rota(180, 0, 0, 1, g)
        S2 = Rota(180, 1, 0, 0, g)
        S3 = Rota(180, 0, 1, 0, g)
        S4 = np.eye(3, 3)
        S = np.vstack((S1, S2, S3, S4))

    if cs == 6:
        S1 = Rota(180, 0, 0, 1, g)
        S2 = np.eye(3, 3)
        S = np.vstack((S1, S2))

    if cs == 7:
        S = np.eye(3, 3)

    return S


def null(A, rcond=None):

    u, s, vh = np.linalg.svd(A, full_matrices=True)
    M, N = u.shape[0], vh.shape[1]
    if rcond is None:
        rcond = np.finfo(s.dtype).eps * max(M, N)
    tol = np.amax(s) * rcond
    num = np.sum(s > tol, dtype=int)
    Q = vh[num:, :].T.conj()
    return Q


def desorientation():
    global cs, Qp, M, M2, D1, D0, S, Dstar, D, D2

    cryststruct()
    gA = M
    gB = M2
    k = 0
    S = Sy(gA)

    D0 = np.zeros((int(np.shape(S)[0] / 3) + 1, 5))
    D1 = np.zeros((int(np.shape(S)[0] / 3) + 1, 3))
    D2 = np.zeros((int(np.shape(S)[0] / 3) + 1, 3))
    Qp = np.zeros((int(np.shape(S)[0] / 3) + 1, 2))

    for i in range(0, np.shape(S)[0], 3):
        In = np.dot(np.array([[S[i, 0], S[i + 1, 0], S[i + 2, 0]], [S[i, 1], S[i + 1, 1], S[i + 2, 1]], [S[i, 2], S[i + 1, 2], S[i + 2, 2]]]), gA)
        A = np.dot(In, np.linalg.inv(gB)) - np.eye(3)
        V = null(A, 0.001).T

        if 0.5 * (np.trace(A + np.eye(3)) - 1) > 1:
            D0[k, 3] = 0
        elif 0.5 * (np.trace(A + np.eye(3)) - 1) < -1:
            D0[k, 3] = 180
        else:
            D0[k, 3] = np.arccos(0.5 * (np.trace(A + np.eye(3)) - 1)) * 180 / np.pi

        if np.abs(D0[k, 3]) < 1e-5:
            D0[k, 0] = 0
            D0[k, 1] = 0
            D0[k, 2] = 0
        else:
            D0[k, 0] = V[0, 0] / np.linalg.norm(V)
            D0[k, 1] = V[0, 1] / np.linalg.norm(V)
            D0[k, 2] = V[0, 2] / np.linalg.norm(V)

        Ds1 = np.dot(np.linalg.inv(gA), np.array([D0[k, 0], D0[k, 1], D0[k, 2]]))
        Ds2 = np.dot(np.linalg.inv(gB), np.array([D0[k, 0], D0[k, 1], D0[k, 2]]))

        if var_uvw() == 1:
            Ds1 = np.dot(np.linalg.inv(D) * 1e-10, Ds1)
            Ds2 = np.dot(np.linalg.inv(D) * 1e-10, Ds2)
        else:
            Ds1 = np.dot(np.linalg.inv(Dstar) * 1e10, Ds1)
            Ds2 = np.dot(np.linalg.inv(Dstar) * 1e10, Ds2)

        D1[k, 0:3] = 100 * Ds1 / np.linalg.norm(Ds1)
        D2[k, 0:3] = 100 * Ds2 / np.linalg.norm(Ds2)

        if D0[k, 2] < 0:
            D0[k, 0] = -D0[k, 0]
            D0[k, 1] = -D0[k, 1]
            D0[k, 2] = -D0[k, 2]
            D1[k, 0] = -D1[k, 0]
            D1[k, 1] = -D1[k, 1]
            D1[k, 2] = -D1[k, 2]

        D0[k, 4] = k
        Qp[k, :] = proj(D0[k, 0], D0[k, 1], D0[k, 2]) * 300

        k = k + 1

    trace()


def trace_misorientation(B):
    global Qp, D1, D0, D2

    i_c = crystal_check()
    ui.misorientation_list.clear()
    sepg = '('
    sepd = ')'
    if var_uvw() == 1:
        sepg = '['
        sepd = ']'
    if Qp.shape[0] > 1:
        ui.misorientation_list.setRowCount(int(np.shape(S)[0] / 3))
        ui.misorientation_list.setColumnCount(3)
        a.plot(B[:-1, 0] + 300, B[:-1, 1] + 300, 's', color='black')
        for l in range(0, int(np.shape(S)[0] / 3)):
            if var_hexa(i_c) == 1 and var_uvw() == 1:
                if ui.axis_checkBox.isChecked():
                    saxe = sepg + str(int((2 * D1[l, 0] - D1[l, 1]) / 3)) + ',' + str(int((2 * D1[l, 1] - D1[l, 0]) / 3)) + ',' + str(int(D1[l, 2])) + sepd
                    a.annotate(saxe, (B[l, 0] + 300, B[l, 1] + 300), size=ui.text_size_entry.text())

                axe1 = QtWidgets.QTableWidgetItem(sepg + str(int((2 * D1[l, 0] - D1[l, 1]) / 3)) + ',' + str(int((2 * D1[l, 1] - D1[l, 0]) / 3)) + ',' + str(int(D1[l, 2])) + sepd)
                axe2 = QtWidgets.QTableWidgetItem(sepg + str(int((2 * D2[l, 0] - D2[l, 1]) / 3)) + ',' + str(int((2 * D2[l, 1] - D2[l, 0]) / 3)) + ',' + str(int(D2[l, 2])) + sepd)
                angle = QtWidgets.QTableWidgetItem(str(np.around(D0[l, 3], decimals=2)))

            else:
                if ui.axis_checkBox.isChecked():
                    saxe = sepg + str(int(D1[l, 0])) + ',' + str(int(D1[l, 1])) + ',' + str(int(D1[l, 2])) + sepd

                    a.annotate(saxe, (B[l, 0] + 300, B[l, 1] + 300), size=ui.text_size_entry.text())

                axe1 = QtWidgets.QTableWidgetItem(sepg + str(int(D1[l, 0])) + ',' + str(int(D1[l, 1])) + ',' + str(int(D1[l, 2])) + sepd)
                axe2 = QtWidgets.QTableWidgetItem(sepg + str(int(D2[l, 0])) + ',' + str(int(D2[l, 1])) + ',' + str(int(D2[l, 2])) + sepd)
                angle = QtWidgets.QTableWidgetItem(str(np.around(D0[l, 3], decimals=2)))

            if ui.numbers_checkBox.isChecked():
                snum = str(int(D0[l, 4]) + 1)
                a.annotate(snum, (B[l, 0] + 300, B[l, 1] + 300), size=ui.text_size_entry.text())

            ui.misorientation_list.setHorizontalHeaderLabels(['Axe 1', 'Axe 2', 'Angle'])
            ui.misorientation_list.setItem(l, 0, axe1)
            ui.misorientation_list.setItem(l, 1, axe2)
            ui.misorientation_list.setItem(l, 2, angle)


def desorientation_clear():
    global Qp
    Qp = np.zeros((0, 2))
    trace()

#######################################################################
#######################################################################
#
# GUI Menu/Dialog
#
#######################################################################


######################################################
#
# Menu
#
##########################################################

###########################################################
#
# Structure
#
##############################################################

def structure(item):
    global x0, var_hexa, d_label_var, e_entry
    item = x0[item - 1]
    ui.abc_entry.setText(str(item[1]) + ',' + str(item[2]) + ',' + str(item[3]))
    ui.alphabetagamma_entry.setText(str(item[4]) + ',' + str(item[5]) + ',' + str(item[6]))
    if eval(item[4]) == 90 and eval(item[5]) == 90 and eval(item[6]) == 120:
        ui.hexa_button.setChecked(True)
        ui.e_entry.setText('2')
    else:
        ui.e_entry.setText('1')
        ui.hexa_button.setChecked(False)


def structure2(item):
    global x0, var_hexa, d_label_var_2, e_entry_2
    item = x0[item - 1]
    ui.abc_entry_2.setText(str(item[1]) + ',' + str(item[2]) + ',' + str(item[3]))
    ui.alphabetagamma_entry_2.setText(str(item[4]) + ',' + str(item[5]) + ',' + str(item[6]))
    if eval(item[4]) == 90 and eval(item[5]) == 90 and eval(item[6]) == 120:
        ui.hexa_button_2.setChecked(True)
        ui.e_entry_2.setText('2')
    else:
        ui.e_entry_2.setText('1')
        ui.hexa_button_2.setChecked(False)

#######################################
#
# Save stereo as png
#
######################################


def image_save():
    filename = QtWidgets.QFileDialog.getSaveFileName(Index, "Save file", "", ".png")
    f = str(filename[0]) + ".png"
    canvas.print_figure(f)


####################
#
# define work space (real or reciprocal) to take tilt/y axis angles into account
#
######################

def ang_work_space():
    if ui.real_space_checkBox.isChecked():
        t_ang = np.float64(ui.image_angle_entry.text())
    else:
        t_ang = np.float64(ui.tilt_angle_entry.text())
    return t_ang

##################################################
#
# Add matplotlib toolbar to zoom and pan
#
###################################################


class NavigationToolbar(NavigationToolbar):
    # only display the buttons we need
    toolitems = [t for t in NavigationToolbar.toolitems if
                 t[0] in ('Pan', 'Zoom')]

    def set_message(self, msg):
        pass


def except_hook(cls, exception, traceback):
    sys.__excepthook__(cls, exception, traceback)

#############################################################
#
# Launch
#
# "


try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8

    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

if __name__ == "__main__":

    app = QtWidgets.QApplication(sys.argv)
    sys.excepthook = except_hook
    QtWidgets.qApp.setApplicationName("Misorientation")
    Index = QtWidgets.QMainWindow()
    ui = misorientationUI.Ui_Misorientation()
    ui.setupUi(Index)
    figure = plt.figure()
    canvas = FigureCanvas(figure)
    ui.mplvl.addWidget(canvas)
    toolbar = NavigationToolbar(canvas, canvas)
    toolbar.setMinimumWidth(601)

# Read structure file

    file_struct = open(os.path.join(os.path.dirname(__file__), 'structure.txt'), "r")
    x0 = []
    for line in file_struct:
        x0.append(list(map(str, line.split())))
    i = 0
    file_struct.close()
    ui.structure_box.addItem(' ')
    ui.structure2_box.addItem(' ')
    for item in x0:
        ui.structure_box.addItem(item[0])
        ui.structure2_box.addItem(item[0])
        i = i + 1

    ui.structure_box.currentIndexChanged.connect(structure)
    ui.structure2_box.currentIndexChanged.connect(structure2)

# Read space_group file


# Ctrl+z shortcut to remove clicked pole

    shortcut = QtWidgets.QShortcut(QtGui.QKeySequence("Ctrl+z"), Index)
    shortcut.activated.connect(undo_click_a_pole)

# Connect dialog boxes and buttons

    ui.actionSave_figure.triggered.connect(image_save)

    ui.button_trace2.clicked.connect(princ2)
    ui.angle_alpha_buttonp.clicked.connect(rot_alpha_p)
    ui.angle_alpha_buttonm.clicked.connect(rot_alpha_m)
    ui.angle_beta_buttonp.clicked.connect(rot_beta_p)
    ui.angle_beta_buttonm.clicked.connect(rot_beta_m)
    ui.angle_z_buttonp.clicked.connect(rot_z_p)
    ui.angle_z_buttonm.clicked.connect(rot_z_m)
    ui.lock_checkButton.stateChanged.connect(lock)
    ui.rot_gm_button.clicked.connect(rotgm)
    ui.rot_gp_button.clicked.connect(rotgp)
    ui.addpole_button.clicked.connect(addpole)
    ui.undo_addpole_button.clicked.connect(undo_addpole)
    ui.sym_button.clicked.connect(addpole_sym)
    ui.undo_sym_button.clicked.connect(undo_sym)
    ui.trace_plan_button.clicked.connect(trace_addplan)
    ui.undo_trace_plan_button.clicked.connect(undo_trace_addplan)
    ui.trace_plan_sym_button.clicked.connect(trace_plan_sym)
    ui.undo_trace_plan_sym_button.clicked.connect(undo_trace_plan_sym)
    ui.reset_view_button.clicked.connect(reset_view)
    figure.canvas.mpl_connect('motion_notify_event', coordinates)
    figure.canvas.mpl_connect('button_press_event', click_a_pole)
    ui.misorientation_button.clicked.connect(desorientation)
    ui.clear_misorientation_button.clicked.connect(desorientation_clear)
    ui.d1_Slider.sliderReleased.connect(dist_restrict1)
    ui.d2_Slider.sliderReleased.connect(dist_restrict2)

# Initialize variables

    dmip = 0
    dmip2 = 0
    var_lock = 0
    ui.lock_checkButton.setChecked(False)
    ui.crystal1_radioButton.setChecked(True)
    ui.color_trace_bleu.setChecked(True)
    ui.color_trace_rouge_2.setChecked(True)
    ui.wulff_button.setChecked(True)
    ui.d_label_var.setText('0')
    ui.d_label_var_2.setText('0')
    ui.text_size_entry.setText('12')
    mpl.rcParams['font.size'] = ui.text_size_entry.text()
    ui.abc_entry.setText('1,1,1')
    ui.alphabetagamma_entry.setText('90,90,90')
    ui.abc_entry_2.setText('1,1,1')
    ui.alphabetagamma_entry_2.setText('90,90,90')
    ui.phi1phiphi2_entry.setText('0,0,0')
    ui.phi1phiphi2_2_entry.setText('0,0,0')

    ui.e_entry.setText('1')
    ui.angle_euler_label.setText(' ')
    ui.size_var.setText('40')
    ui.e_entry_2.setText('1')
    ui.angle_alpha_entry.setText('5')
    ui.angle_beta_entry.setText('5')
    ui.angle_z_entry.setText('5')
    ui.angle_beta_entry.setText('5')
    ui.angle_z_entry.setText('5')
    ui.rot_g_entry.setText('5')
    ui.tilt_angle_entry.setText('0')
    ui.image_angle_entry.setText('0')
    a = figure.add_subplot(111)
    tilt_axes()
    wulff()
    Index.show()
    sys.exit(app.exec_())
